#include "MainWindow.h"
#include "MDIWindow.h"
#include "GLMDIWindow.h"
#include "VolumeFile.h"
#include "GLView.h"
#include "VoxelInfoDlg.h"
#include "MeshFilteringInfoDlg.h"
#include "FEModelSaveAsImageDlg.h"
#include "MeshFilteringDlg.h"
#include "MultiLineEditDlg.h"
#include <qfile.h>
#include <qdir.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qradiobutton.h>
#include <qcombobox.h>
#include <qimage.h>
#include <qtoolbutton.h>
#include <qmessagebox.h>


////////////////////////////////////////////////////////////////////////////
//
// GLMDIWindow
//

GLMDIWindow::GLMDIWindow( QWidget *parent, const char *name, WFlags f )
	: QMainWindow( parent, name, f )
{
	m_pVolume = NULL;
	m_pMDIWindow = NULL;
	m_pVoxelInfoDlg = NULL;

	// OpenGL view
	m_pGLView = new CGLView( this );
	ASSERT( m_pGLView );

	setFocusProxy( m_pGLView );
	setCentralWidget( m_pGLView );

//	setBackgroundMode( NoBackground );
} // GLMDIWindow

GLMDIWindow::~GLMDIWindow()
{
	if ( m_pMDIWindow )
		m_pMDIWindow->unsetOpenGLView();

	if ( m_pGLView )
	{
		delete m_pGLView;
		m_pGLView = NULL;
	}

	if ( m_pVoxelInfoDlg )
	{
		delete m_pVoxelInfoDlg;
		m_pVoxelInfoDlg = NULL;
	}
} // ~GLMDIWindow

CGLView *GLMDIWindow::openglView()
{
	if ( m_pGLView )
		return m_pGLView;

	return NULL;
} // openglView

bool GLMDIWindow::init( CVolume *pVolume )
{
	m_pVolume = pVolume;
	ASSERT( m_pVolume );

	if ( !m_pVolume->findCurrentVisibleSurface() )
		return false;

	if ( !m_pGLView->init( m_pVolume ) )
		return false;

	QString fileName = m_pVolume->volumeFileName();
	if ( fileName.isEmpty() )
		fileName = "unnamed FE Model";
		
	setCaption( tr( "3D Model : " ) + fileName );

	resize( 512, 512 );

	return true;
} // init

CVolume *GLMDIWindow::volume()
{
	return m_pVolume;
} // volume

void GLMDIWindow::setMDIWindow( MDIWindow *pMDIWindow )
{
	m_pMDIWindow = pMDIWindow;
} // setMDIWindow

bool GLMDIWindow::hasVoxelInfoDlg()
{
	if ( m_pVoxelInfoDlg )
		return true;
	else
		return false;
} // hasVoxelInfoDlg

void GLMDIWindow::setVoxelInfoDlg( CVoxelInfoDlg *pVoxelInfoDlg )
{
	m_pVoxelInfoDlg = pVoxelInfoDlg;
	ASSERT( m_pVoxelInfoDlg );
} // setVoxelInfoDlg

void GLMDIWindow::unsetVoxelInfoDlg()
{
	m_pVoxelInfoDlg = NULL;
} // unsetVoxelInfoDlg

bool GLMDIWindow::resetSegmentation()
{
	if ( !m_pVolume )
		return false;

	if ( !m_pVolume->findCurrentVisibleSurface( true ) )
		return false;

	m_pVolume->setNumFilterVoxel( 0 );

	return true;
} // resetSegmentation

// slot
bool GLMDIWindow::slotMeshFiltering()
{
	if ( !m_pVolume )
		return false;

	int oldFilter = m_pVolume->numFilterVoxel();

	CMeshFilteringDlg *pDlg = new CMeshFilteringDlg( this, NULL, true );
	if ( pDlg->init( this ) )
	{
		if ( pDlg->exec() != QDialog::Accepted )
			return false;

		DWORD filter;

		// contact option
		if ( pDlg->IDC_RADIO_SURFACE->isOn() )
			m_pVolume->setSurfaceContact( true );
		else
			m_pVolume->setSurfaceContact( false );

		// filtering option
		if ( pDlg->IDC_RADIO_NUMBER->isOn() )
		{
			filter = pDlg->IDC_EDIT_FILTER->text().toULong();
		}
		else
		{
			float volume = pDlg->IDC_EDIT_FILTER->text().toFloat();
			float voxelVolume = m_pVolume->voxelVolume();

			filter = (DWORD) (volume / voxelVolume );
		}

		if ( filter <= 0 )
			filter = 0;
		if ( filter >= m_pVolume->totalNumVoxel() )
			filter = 10;

		if ( filter == m_pVolume->numFilterVoxel() )
			return true;

		m_pVolume->setNumFilterVoxel( filter );

		bool bRet = m_pVolume->calculateNumCurrentBone( true );
		if ( !bRet )
		{
			m_pVolume->setNumFilterVoxel( oldFilter );
			return false;
		}

		CMeshFilteringInfoDlg *pDlg2 = new CMeshFilteringInfoDlg( this, NULL, true );
		if ( pDlg2->init( m_pVolume->numCurrentBone(), m_pVolume->numFilteredBone() ) )
		{
			pDlg2->show();
			return true;
		}
		else
			return false;
	}
		return false;
} // slotMeshFiltering

// slot
bool GLMDIWindow::slotMeshFilterInfo()
{
	WFlags f = Qt::WType_TopLevel | Qt::WStyle_MinMax;
#ifdef Q_WS_WIN
	f |= Qt::WStyle_SysMenu;
#endif

	CMultiLineEditDlg dlg( this, NULL, true, f );
	
	QString strFileName = QDir::homeDirPath() + "/.mesh_filter.tmp";

	if ( !QFile::exists( strFileName ) )
	{
		QMessageBox::warning( this, tr( "Error on File" ),
			tr( "Can't find filtering file" ) );

		m_pVolume->setMeshFilter( false );
		return false;
	}

	if ( dlg.init( strFileName, "Mesh Filter Report" ) )
		dlg.show();

	return true;
} // slotMeshFilterInfo

// slot
bool GLMDIWindow::slotSaveAsImage()
{
	if ( !m_pVolume )
		return false;

	CFEModelSaveAsImageDlg dlg( this, NULL, true );
	if ( dlg.init() )
	{
		if ( dlg.exec() != QDialog::Accepted )
			return false;

		// check file existance
		QString strFileName = dlg.IDC_EDIT_FILENAME->text();
		QString strFileType = dlg.IDC_COMBO_FILEFORMAT->currentText();
		QString strMsg = strFileName
			+ "This file already exists.\n"
			+ "Replace existing file ?";

		if ( QFile::exists( strFileName ) )
		{
			/*
			QMessageBox mb( tr( "Warning" ), strMsg,
				QMessageBox::Warning,
				QMessageBox::Yes,
				QMessageBox::No | QMessageBox::Default,
				NULL );

			mb.setButtonText( QMessageBox::Yes, "Ok" );
			mb.setButtonText( QMessageBox::No, "Cancel" );

			int ret = mb.exec();
			if ( ret == QMessageBox::Yes )
				QFile::remove( strFileName );
			else if( ret == QMessageBox::No )
				return false;
			*/

			QFile::remove( strFileName );
		}
		
		/*
		bool bTitle = false;
		bool bGrid = false;
		bool bBoundingBox = false;

		if ( dlg.IDC_CHECK_TITLE->isChecked() )
			bTitle = true;
		if ( dlg.IDC_CHECK_GRIDLINE->isChecked() )
			bGrid = true;
		if ( dlg.IDC_CHECK_BOUNDINGBOX->isChecked() )
			bBoundingBox = true;

		// do not update view
		m_pGLView->setShowTitle( bTitle, false );
		m_pGLView->setShowGridLine( bGrid, false );
		m_pGLView->setShowBoundingBox( bBoundingBox, false );

		m_pGLView->updateView( false );
		*/

		QImage image = m_pGLView->imagePixmap().convertToImage();
		if ( !image.save( strFileName, strFileType ) )
			return false;

		/*
		m_pGLView->setShowTitle( true, false );
		m_pGLView->setShowGridLine( true, false );
		m_pGLView->setShowBoundingBox( true, false );
		*/

		// check title, bounding box, grid line
		/*
		if ( m_pGLView->isTitleShow() )
			g_pMainWindow->m_pTitleShowHide->setOn( true );
		else
			g_pMainWindow->m_pTitleShowHide->setOn( false );

		if ( m_pGLView->isBoundingBoxShow() )
			g_pMainWindow->m_pBoundingBoxShowHide->setOn( true );
		else
			g_pMainWindow->m_pBoundingBoxShowHide->setOn( false );

		if ( m_pGLView->isGridLineShow() )
			g_pMainWindow->m_pGridLineShowHide->setOn( true );
		else
			g_pMainWindow->m_pGridLineShowHide->setOn( false );


		m_pGLView->updateView();
		*/

		QMessageBox::information( this, "Save as Image",
			tr( "Saving as Image Done..." ) );
	}

	return true;
} // slotSaveAsImage

bool GLMDIWindow::slotAnalysisVolume()
{
	if ( m_pVolume->analysisVolume() )
		return true;
	else
		return false;
}

// virtual
void GLMDIWindow::closeEvent( QCloseEvent *e )
{
	if ( hasVoxelInfoDlg() )
	{
		e->ignore();
		return;
	}
	else
		e->accept();
} // closeEvent
