#include "define.h"
#include "VolumeFile.h"
#include "MainWindow.h"
#include "MDIWindow.h"
#include "GLMDIWindow.h"
#include "RenderMDIWindow.h"
#include "MontageMDIWindow.h"
#include "MarchingCubeMDIWindow.h"
#include "ImportStackWizard.h"
#include "SegmentationDlg.h"
#include "PlanView.h"
#include "GLView.h"
#include "VoxelInfoDlg.h"
#include "VolumeInfoDlg.h"
#include "MeshFilteringDlg.h"
#include "AboutDlg.h"

#include <qapplication.h>
#include <qaccel.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <qprogressbar.h>
#include <qpopupmenu.h>
#include <qaction.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qdir.h>
#include <qstringlist.h>
#include <qvbox.h>

#include "toolbar_xpm.h"


CMainWindow *g_pMainWindow = NULL;


////////////////////////////////////////////////////////////////////////////
//
// CMainWindow
//

CMainWindow::CMainWindow( QWidget* parent, const char* name, WFlags f )
	: QMainWindow( parent, name, f )
{
	g_pMainWindow = this;

	m_nRecentFiles = 0;		// number of recent files

	m_pFileMenu = NULL;
	m_p3DModelMenu = NULL;

	setCaption("MGIBA");

	QVBox* vb = new QVBox( this );
	vb->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
	ws = new QWorkspace( vb );

	ws->setBackgroundMode( PaletteDark );
	setCentralWidget( vb );

	setMenuBar();
	setToolBar();
	setStatusBar();
	changeMenuAndToolBar( NONE_VIEW );

	connect( ws, SIGNAL( windowActivated( QWidget * ) ),
		this, SLOT( slotWindowActivated( QWidget * ) ) );
} // CMainWindow

CMainWindow::~CMainWindow()
{
} // ~CMainWindow

void CMainWindow::init()
{
	/****************/
	/* recent files */
	/****************/

	QFile f1( QDir::homeDirPath() + RC_RECENT_FILES );

	// recent files
	if ( !QFile::exists( QDir::homeDirPath() + RC_RECENT_FILES ) )	// global_define.h
	{
		QDir d;
		d.mkdir( QDir::homeDirPath() + RC_DIRECTORY );

		if ( f1.open( IO_WriteOnly ) )
		{
			QTextStream ts( &f1 );

			ts << "# " << PROGRAM_NAME << " Ver. " << PROGRAM_VERSION << endl;
			ts << "# Recent Files" << endl;
			ts << "0" << endl;

			f1.close();
			ts.unsetDevice();
		}
	}
	else
	{
		if ( f1.open( IO_ReadOnly ) )
		{
			QTextStream ts( &f1 );

			ts.readLine();
			ts.readLine();
			QString str;
			// number of file : max number of recent files is 4
			str = ts.readLine();
			m_nRecentFiles = str.toInt();
			for ( int i=0 ; i<m_nRecentFiles ; i++ )
				m_strRecentFiles[i] = ts.readLine();

			f1.close();
			ts.unsetDevice();
		}
	}

	// window position
	QFile f2( QDir::homeDirPath() + RC_WINDOW_POSITION );
	if ( f2.open( IO_ReadOnly ) )
	{
		int x, y;
		int width, height;

		QTextStream ts( &f2 );

		ts.readLine();
		ts.readLine();
		ts >> x >> y;
		ts >> width >> height;

		QPoint p( x, y );
		QSize s( width, height );

		f2.close();
		ts.unsetDevice();

        resize( s );		// restore size
        move( p );			// restore position
	}
	else
		showMaximized();	// maximize
} // init

QMainWindow* CMainWindow::newDoc( int type )
{
	QMainWindow *w;

	if ( type == PLAN_VIEW )
	{
		w = new MDIWindow( ws, 0, WDestructiveClose );
		w->setCaption( "unnamed volume" );
	}
	else if ( type == OPENGL_VIEW )
	{
		w = new GLMDIWindow( ws, 0, WDestructiveClose );
	}
	else if ( type == RENDER_VIEW )
	{
		w = new RenderMDIWindow( ws, 0, WDestructiveClose );
	}
	else if ( type == MONTAGE_VIEW )
	{
		w = new MontageMDIWindow( ws, 0, WDestructiveClose );
	}
	else if ( type == MCUBE_VIEW )
	{
		w = new MarchingCubeMDIWindow( ws, 0, WDestructiveClose );
	}

    return w;
} // newDoc

void CMainWindow::closeMDIWindow( QString str )
{
	for ( QStringList::Iterator it = m_listVolume.begin(); it != m_listVolume.end(); ++it )
	{
		if ( str == *it )
		{
			m_listVolume.remove( *it );
			return;
		}
	}
} // closeMDIWindow

void CMainWindow::setMenuBar()
{
	/*************/
	/* main menu */
	/*************/

	// file menu
	m_pFileMenu = new QPopupMenu;
	m_pFileMenu->setCheckable( true );
    connect( m_pFileMenu, SIGNAL( aboutToShow() ),
		this, SLOT( slotFileMenuAboutToShow() ) );
	slotFileMenuAboutToShow();	// important to activate accel key..^^

	// view menu
	m_pViewMenu = new QPopupMenu;
	m_pViewMenu->setCheckable( true );
	connect( m_pViewMenu, SIGNAL( aboutToShow() ),
		this, SLOT( slotViewMenuAboutToShow() ) );

	// 3d model menu
	m_p3DModelMenu = new QPopupMenu;
	m_p3DModelMenu->setCheckable( true );
	connect( m_p3DModelMenu, SIGNAL( aboutToShow() ),
		this, SLOT( slot3DModelMenuAboutToShow() ) );

	// image menu
	m_pImageMenu = new QPopupMenu;
	connect( m_pImageMenu, SIGNAL( aboutToShow() ),
		this, SLOT( slotImageMenuAboutToShow() ) );

	// help menu
	m_pHelpMenu = new QPopupMenu;
	connect( m_pHelpMenu, SIGNAL( aboutToShow() ),
		this, SLOT( slotHelpMenuAboutToShow() ) );

	m_nFileMenuId = menuBar()->insertItem( tr( "&File" ), m_pFileMenu );
	m_nViewMenuId = menuBar()->insertItem( tr( "&View" ), m_pViewMenu );
	m_n3DModelMenuId = menuBar()->insertItem( tr( "3D &Model" ), m_p3DModelMenu );
	m_nImageMenuId = menuBar()->insertItem( tr( "&Image" ), m_pImageMenu );
	m_nHelpMenuId = menuBar()->insertItem( tr( "&Help" ), m_pHelpMenu );


	/**************/
	/* popup menu */
	/**************/

	// export popup
	m_pExportPopup = new QPopupMenu;
	connect( m_pExportPopup, SIGNAL( aboutToShow() ),
		this, SLOT( slotExportPopupAboutToShow() ) );

	// recent files popup
	m_pRecentFilesPopup = new QPopupMenu;
	connect( m_pRecentFilesPopup, SIGNAL( aboutToShow() ),
		this, SLOT( slotRecentFilesPopupAboutToShow() ) );

	// tool popup
	m_pToolPopup = new QPopupMenu;
	m_pToolPopup->setCheckable( true );
	connect( m_pToolPopup, SIGNAL( aboutToShow() ),
		this, SLOT( slotToolPopupAboutToShow() ) );

	// image palette popup
	m_pImagePalettePopup = new QPopupMenu;
	m_pImagePalettePopup->setCheckable( true );
	connect( m_pImagePalettePopup, SIGNAL( aboutToShow() ),
		this, SLOT( slotImagePalettePopupAboutToShow() ) );

	// pre defined view popup
	m_pPreDefinedViewPopup = new QPopupMenu;
	connect( m_pPreDefinedViewPopup, SIGNAL( aboutToShow() ),
		this, SLOT( slotPreDefinedViewPopupAboutToShow() ) );
} // setMenuBar

void CMainWindow::setToolBar()
{
	/****************/
	/* main toolbar */
	/****************/

	m_pMainToolBar = new QToolBar( this );

	QPixmap openIcon = QPixmap( fileOpen_xpm );
	new QToolButton( openIcon, "Open File", QString::null,
			this, SLOT( slotOpen() ), m_pMainToolBar, "open file" );

	QPixmap saveIcon = QPixmap( fileSave_xpm );
	new QToolButton( saveIcon, "Save File", QString::null,
			this, SLOT( slotSave() ), m_pMainToolBar, "save file" );

	QPixmap importStackIcon = QPixmap( importStack_xpm );
	new QToolButton( importStackIcon, "Import Stack", QString::null,
			this, SLOT( slotImportStack() ), m_pMainToolBar, "import stack" );


	/*********************/
	/* plan view toolbar */
	/*********************/

	m_pPlanViewToolBar = new QToolBar( this );

	// segmentation
	QPixmap segmentationShowHideIcon = QPixmap( segmentationShowHide_xpm );
	m_pSegmentationShowHide
		= new QToolButton( segmentationShowHideIcon, "Show/Hide Segmentation", QString::null,
			this, SLOT( slotShowSegmentation() ), m_pPlanViewToolBar, "show_hide segmentation" );
	m_pSegmentationShowHide->setToggleButton( true );

	QPixmap segmentationIcon = QPixmap( segmentation_xpm );
	m_pSegmentation = new QToolButton( segmentationIcon, "Segmentation", QString::null,
			this, SLOT( slotSegmentation() ), m_pPlanViewToolBar, "segmentation" );

	m_pPlanViewToolBar->addSeparator();

	// region of interest
	QPixmap ROIIcon = QPixmap( ROI_xpm );
	m_pROI = new QToolButton( ROIIcon, "ROI", QString::null,
				this, SLOT( slotROI() ), m_pPlanViewToolBar, "ROI" );
	m_pROI->setToggleButton( true );

	QPixmap clearROIIcon = QPixmap( clearROI_xpm );
	m_pROIShowHide = new QToolButton( clearROIIcon, "Clear ROI", QString::null,
			this, SLOT( slotROIClearROI() ), m_pPlanViewToolBar, "clear ROI" );

	m_pPlanViewToolBar->addSeparator();

	// measure
	QPixmap measureLengthIcon = QPixmap( measureLength_xpm );
	m_pMeasureLength
		= new QToolButton( measureLengthIcon, "Measure", QString::null,
				this, SLOT( slotMeasureLength() ), m_pPlanViewToolBar, "measure length" );
	m_pMeasureLength->setToggleButton( true );

	QPixmap measureAngleIcon = QPixmap( measureAngle_xpm );
	m_pMeasureAngle
		= new QToolButton( measureAngleIcon, "Angle", QString::null,
				this, SLOT( slotMeasureAngle() ), m_pPlanViewToolBar, "measure angle" );
	m_pMeasureAngle->setToggleButton( true );

	/*
	QPixmap measureRegionIcon = QPixmap( measureRegion_xpm );
	m_pMeasureRegion
		= new QToolButton( measureRegionIcon, "Region", QString::null,
				this, SLOT( slotMeasureRegion() ), m_pPlanViewToolBar, "measure region" );
	m_pMeasureRegion->setToggleButton( true );
	*/

	QPixmap measureShowHideIcon = QPixmap( measureShowHide_xpm );
	m_pMeasureLineShowHide
		= new QToolButton( measureShowHideIcon, "Show/Hide Measure Line", QString::null,
				this, SLOT( slotShowMeasureLine() ), m_pPlanViewToolBar, "show_hide measure line" );
	m_pMeasureLineShowHide->setToggleButton( true );

	QPixmap measureClearLineIcon = QPixmap( measureClearLine_xpm );
	new QToolButton( measureClearLineIcon, "Clear Measure Line", QString::null,
			this, SLOT( slotMeasureClearLine() ), m_pPlanViewToolBar, "measure clear" );

	m_pPlanViewToolBar->addSeparator();

	// fill & erase segmentation
	QPixmap fillSegmentationIcon = QPixmap( fillSegmentation_xpm );
	m_pFillSegmentation 
		= new QToolButton( fillSegmentationIcon, "Fill", QString::null,
				this, SLOT( slotFillSegmentation() ), m_pPlanViewToolBar, "fill" );
	m_pFillSegmentation->setToggleButton( true );

	QPixmap eraseSegmentationIcon = QPixmap( eraseSegmentation_xpm );
	m_pEraseSegmentation 
		= new QToolButton( eraseSegmentationIcon, "Erase", QString::null,
				this, SLOT( slotEraseSegmentation() ), m_pPlanViewToolBar, "erase" );
	m_pEraseSegmentation->setToggleButton( true );

	QPixmap update3DModelIcon = QPixmap( update3DModel_xpm );
	new QToolButton( update3DModelIcon, "Update 3D-Model", QString::null,
		this, SLOT( slotUpdate3DModel() ), m_pPlanViewToolBar, "update 3d model" );


	/****************/
	/* view toolbar */
	/****************/

	m_pViewToolBar = new QToolBar( this );

	QPixmap defaultViewIcon = QPixmap( defaultView_xpm );
	new QToolButton( defaultViewIcon, "Default View", QString::null,
			this, SLOT( slotDefaultView() ), m_pViewToolBar, "default view" );

	QPixmap zoomAllIcon = QPixmap( zoomAll_xpm );
	new QToolButton( zoomAllIcon, "Zoom All", QString::null,
			this, SLOT( slotZoomAll() ), m_pViewToolBar, "zoom all" );

	QPixmap zoomInIcon = QPixmap( zoomIn_xpm );
	m_pZoomIn = new QToolButton( zoomInIcon, "Zoom In", QString::null,
			this, SLOT( slotZoomIn() ), m_pViewToolBar, "zoom in" );

	QPixmap zoomOutIcon = QPixmap( zoomOut_xpm );
	m_pZoomOut = new QToolButton( zoomOutIcon, "Zoom Out", QString::null,
			this, SLOT( slotZoomOut() ), m_pViewToolBar, "zoom out" );

	m_pViewToolBar->addSeparator();

	QPixmap plusXIcon = QPixmap( plusX_xpm );
	new QToolButton( plusXIcon, "Plus X", QString::null,
			this, SLOT( slotPlusX() ), m_pViewToolBar, "plus x" );

	QPixmap minusXIcon = QPixmap( minusX_xpm );
	new QToolButton( minusXIcon, "Minux X", QString::null,
			this, SLOT( slotMinusX() ), m_pViewToolBar, "minus x" );

	QPixmap plusYIcon = QPixmap( plusY_xpm );
	new QToolButton( plusYIcon, "Plus Y", QString::null,
			this, SLOT( slotPlusY() ), m_pViewToolBar, "plus y" );

	QPixmap minusYIcon = QPixmap( minusY_xpm );
	new QToolButton( minusYIcon, "Minus Y", QString::null,
			this, SLOT( slotMinusY() ), m_pViewToolBar, "minus y" );

	QPixmap plusZIcon = QPixmap( plusZ_xpm );
	new QToolButton( plusZIcon, "Plus Z", QString::null,
			this, SLOT( slotPlusZ() ), m_pViewToolBar, "plus z" );

	QPixmap minusZIcon = QPixmap( minusZ_xpm );
	new QToolButton( minusZIcon, "Minus Z", QString::null,
			this, SLOT( slotMinusZ() ), m_pViewToolBar, "minus z" );


	/********************/
	/* FE-Model toolbar */
	/********************/
	
	m_pFEModelToolBar = new QToolBar( this );

	QPixmap whiteToBlackColorIcon = QPixmap( whiteToBlackColor_xpm );
	m_pWhiteToBlackColor = new QToolButton( whiteToBlackColorIcon, "White to Black Color", QString::null,
			this, SLOT( slotImagePaletteWhiteToBlackColor() ), m_pFEModelToolBar, "white to black color" );
	m_pWhiteToBlackColor->setToggleButton( true );

	QPixmap blackToWhiteColorIcon = QPixmap( blackToWhiteColor_xpm );
	m_pBlackToWhiteColor = new QToolButton( blackToWhiteColorIcon, "Black to White Color", QString::null,
			this, SLOT( slotImagePaletteBlackToWhiteColor() ), m_pFEModelToolBar, "black to white color" );
	m_pBlackToWhiteColor->setToggleButton( true );

	QPixmap singleColorIcon = QPixmap( singleColor_xpm );
	m_pSingleColor = new QToolButton( singleColorIcon, "Single Color", QString::null,
			this, SLOT( slotImagePaletteSingleColor() ), m_pFEModelToolBar, "single color" );
	m_pSingleColor->setToggleButton( true );

	QPixmap indexColorIcon = QPixmap( indexColor_xpm );
	m_pIndexColor = new QToolButton( indexColorIcon, "Index Color", QString::null,
			this, SLOT( slotImagePaletteIndexColor() ), m_pFEModelToolBar, "index color" );
	m_pIndexColor->setToggleButton( true );

	m_pFEModelToolBar->addSeparator();

	QPixmap selectSingleColorIcon = QPixmap( selectSingleColor_xpm );
	new QToolButton( selectSingleColorIcon, "Select Single Color", QString::null,
			this, SLOT( slotImagePaletteSelectSingleColor() ), m_pFEModelToolBar, "select single color" );

	QPixmap selectIndexRangeIcon = QPixmap( selectIndexRange_xpm );
	new QToolButton( selectIndexRangeIcon, "Select Index Range", QString::null,
			this, SLOT( slotImagePaletteSelectIndexRange() ), m_pFEModelToolBar, "select index range" );

	m_pFEModelToolBar->addSeparator();

	QPixmap filteredShowHideIcon = QPixmap( filteredShowHide_xpm );
	m_pFilteredShowHide= new QToolButton( filteredShowHideIcon, "Show/Hide Filtered Voxel", QString::null,
			this, SLOT( slotFilteredShowHide() ), m_pFEModelToolBar, "show_hide filtered voxel" );
	m_pFilteredShowHide->setToggleButton( true );

	m_pFEModelToolBar->addSeparator();

	QPixmap titleShowHideIcon = QPixmap( titleShowHide_xpm );
	m_pTitleShowHide = new QToolButton( titleShowHideIcon, "Show/Hide Title", QString::null,
			this, SLOT( slotTitleShowHide() ), m_pFEModelToolBar, "show_hide title" );
	m_pTitleShowHide->setToggleButton( true );

	QPixmap boundingBoxShowHideIcon = QPixmap( boundingBoxShowHide_xpm );
	m_pBoundingBoxShowHide = new QToolButton( boundingBoxShowHideIcon, "Show/Hide Bounding Box", QString::null,
			this, SLOT( slotBoundingBoxShowHide() ), m_pFEModelToolBar, "show_hide bounding box" );
	m_pBoundingBoxShowHide->setToggleButton( true );

	QPixmap gridLineShowHideIcon = QPixmap( gridLineShowHide_xpm );
	m_pGridLineShowHide = new QToolButton( gridLineShowHideIcon, "Show/Hide Grid Line", QString::null,
			this, SLOT( slotGridLineShowHide() ), m_pFEModelToolBar, "show_hide grid line" );
	m_pGridLineShowHide->setToggleButton( true );
} // setToolBar

void CMainWindow::setStatusBar()
{
	statusBar()->message( tr( "Ready" ) );
} // setStatusBar

// slot
void CMainWindow::slotFileMenuAboutToShow()
{
	QWidget *widget;
	int nType;

	if ( ws )
	{
		widget = ws->activeWindow();

		// check view type
		if ( widget && widget->isA( "MDIWindow" ) )
			nType = PLAN_VIEW;
		else if ( widget && widget->isA( "GLMDIWindow" ) )
			nType = OPENGL_VIEW;
		else if ( widget && widget->isA( "RenderMDIWindow" ) )
			nType = RENDER_VIEW;
		else if ( widget && widget->isA( "MontageMDIWindow" ) )
			nType = MONTAGE_VIEW;
		else if ( widget && widget->isA( "MarchingCubeMDIWindow" ) )
			nType = MCUBE_VIEW;
	}


	// draw file menu
	m_pFileMenu->clear();

	QPixmap openIcon = QPixmap( fileOpen_xpm );
	QPixmap saveIcon = QPixmap( fileSave_xpm );
	QPixmap importStackIcon = QPixmap( importStack_xpm );


	m_pFileMenu->insertItem( openIcon, tr( "&Open" ),
		this, SLOT( slotOpen() ), CTRL+Key_O );

	m_pFileMenu->insertItem( saveIcon, tr( "&Save" ),
		this, SLOT( slotSave() ), CTRL+Key_S );

	m_pFileMenu->insertSeparator();

	m_pFileMenu->insertItem( importStackIcon, tr( "&Import Stack..." ),
		this, SLOT( slotImportStack() ), CTRL+Key_I );

	m_pFileMenu->insertSeparator();

	if ( ws )
	{
		if ( nType == PLAN_VIEW && widget &&
			 ((MDIWindow *)widget)->hasOpenGLView() )
		{
			m_pFileMenu->insertItem( tr( "Export" ), m_pExportPopup );

			m_pFileMenu->insertSeparator();
		}
		if ( nType == OPENGL_VIEW )
		{
			m_pFileMenu->insertItem( tr( "Export" ), m_pExportPopup );
		
			m_pFileMenu->insertSeparator();
		}
		if ( nType == MCUBE_VIEW )
		{
			m_pFileMenu->insertItem( tr( "Export" ), m_pExportPopup );

			m_pFileMenu->insertSeparator();
		}
	}

	// recent files
	if ( m_nRecentFiles != 0 )
	{
		m_pFileMenu->insertItem( tr( "Recent Files" ), m_pRecentFilesPopup ); 
		m_pFileMenu->insertSeparator();
	}

	// exit
	m_pFileMenu->insertItem( tr( "E&xit" ),
		qApp, SLOT( closeAllWindows() ), CTRL+Key_X );	// QAccel::stringToKey( "Ctrl+Key_X" ) );
} // slotFileMenuAboutToShow

// slot
void CMainWindow::slotViewMenuAboutToShow()
{
	m_pViewMenu->clear();

	m_pViewMenu->insertItem( tr( "&Toolbars" ), m_pToolPopup );

	QWidget *widget;
	if ( ws )
		widget = ws->activeWindow();

	int id;

	// plan view
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;

		m_pViewMenu->insertSeparator();

		QPixmap zoomInIcon = QPixmap( zoomIn_xpm );
		m_pViewMenu->insertItem( zoomInIcon, tr( "Zoom &In" ), this, SLOT( slotZoomIn() ) );
		QPixmap zoomOutIcon = QPixmap( zoomOut_xpm );
		m_pViewMenu->insertItem( zoomOutIcon, tr( "Zoom &Out" ), this, SLOT( slotZoomOut() ) );
		QPixmap zoomAllIcon = QPixmap( zoomAll_xpm );
		m_pViewMenu->insertItem( zoomAllIcon, tr( "Zoom &All" ), this, SLOT( slotZoomAll() ) );

		m_pViewMenu->insertSeparator();

		id = m_pViewMenu->insertItem( tr( "X-View" ), this, SLOT( slotXView() ) );
		m_pViewMenu->setItemChecked( id, m->planView()->checkViewType( 1 ) );
		id = m_pViewMenu->insertItem( tr( "Y-View" ), this, SLOT( slotYView() ) );
		m_pViewMenu->setItemChecked( id, m->planView()->checkViewType( 2 ) );
		id = m_pViewMenu->insertItem( tr( "Z-View" ), this, SLOT( slotZView() ) );
		m_pViewMenu->setItemChecked( id, m->planView()->checkViewType( 3 ) );

		m_pViewMenu->insertSeparator();

		m_pViewMenu->insertItem( tr( "Next Slice" ), this, SLOT( slotNextSlice() ) );
		m_pViewMenu->insertItem( tr( "Previous Slice" ), this, SLOT( slotPreviousSlice() ) );
	}
	// 3d model view
	else if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		m_pViewMenu->insertSeparator();

		m_pViewMenu->insertItem( tr( "Image Palette" ), m_pImagePalettePopup );

		m_pViewMenu->insertSeparator();

		QPixmap zoomAllIcon = QPixmap( zoomAll_xpm );
		m_pViewMenu->insertItem( zoomAllIcon, tr( "Zoom All" ),
			this, SLOT( slotZoomAll() ) );
		QPixmap defaultViewIcon = QPixmap( defaultView_xpm );
		m_pViewMenu->insertItem( defaultViewIcon, tr( "Default View" ),
			this, SLOT( slotDefaultView() ) );

		m_pViewMenu->insertSeparator();

		m_pViewMenu->insertItem( tr( "Pre-Defined Views" ), m_pPreDefinedViewPopup );
	}
} // slotViewMenuAboutToShow

void CMainWindow::slotExportPopupAboutToShow()
{
	m_pExportPopup->clear();

	m_pExportPopup->insertTearOffHandle();

	QWidget *widget;
	if ( ws )
		widget = ws->activeWindow();

	if ( widget &&
		 ( widget->isA( "GLMDIWindow" ) ||
		   ( widget->isA( "MDIWindow" ) && ((MDIWindow *)widget)->hasOpenGLView() ) )
	   )
	{
		m_pExportPopup->insertItem( tr( "To Ansys..." ), this, SLOT( slotExportToAnsys() ) );

		m_pExportPopup->insertSeparator();

		m_pExportPopup->insertItem( tr( "To STL..." ), this, SLOT( slotExportToSTLAscii() ) );
		m_pExportPopup->insertItem( tr( "To ACAD Script..." ), this, SLOT( slotExportToAcadScript() ) );
		m_pExportPopup->insertItem( tr( "To ACAD DXF..." ), this, SLOT( slotExportToAcadDxf() ) );
	}
	else if ( widget && widget->isA( "MarchingCubeMDIWindow" ) )
	{
		m_pExportPopup->insertItem( tr( "To STL..." ), this, SLOT( slotExportToSTLAscii() ) );
	}
} // slotExportPopupAboutToShow

void CMainWindow::slotRecentFilesPopupAboutToShow()
{
	m_pRecentFilesPopup->clear();

	m_pRecentFilesPopup->insertTearOffHandle();

	for ( int i=0 ; i<m_nRecentFiles ; i++ )
	{
		switch( i )
		{
		case 0:
			m_pRecentFilesPopup->insertItem( "&1 " + m_strRecentFiles[i],
				this , SLOT( slotRecentFile1() ) );
			break;
		case 1:
			m_pRecentFilesPopup->insertItem( "&2 " + m_strRecentFiles[i],
				this , SLOT( slotRecentFile2() ) );
			break;
		case 2:
			m_pRecentFilesPopup->insertItem( "&3 " + m_strRecentFiles[i],
				this , SLOT( slotRecentFile3() ) );
			break;
		case 3:
			m_pRecentFilesPopup->insertItem( "&4 " + m_strRecentFiles[i],
				this , SLOT( slotRecentFile4() ) );
			break;
		}
	}
} // slotRecentFilesPopupAboutToShow

// slot
void CMainWindow::slotToolPopupAboutToShow()
{
	m_pToolPopup->clear();
	
	m_pToolPopup->insertTearOffHandle();

	int id;
	id = m_pToolPopup->insertItem( tr( "&Main" ), this, SLOT( slotToolMainToolBar() ) );
	if ( m_pMainToolBar )
		m_pToolPopup->setItemChecked( id, m_pMainToolBar->isVisible() );

	id = m_pToolPopup->insertItem( tr( "&Plan View" ), this, SLOT( slotToolSegMeasureToolBar() ) );
	if ( m_pPlanViewToolBar )
		m_pToolPopup->setItemChecked( id, m_pPlanViewToolBar->isVisible() );

	id = m_pToolPopup->insertItem( tr( "&View" ), this, SLOT( slotToolViewToolBar() ) );
	if ( m_pViewToolBar )
		m_pToolPopup->setItemChecked( id, m_pViewToolBar->isVisible() );

	id = m_pToolPopup->insertItem( tr( "&FE-Model" ), this, SLOT( slotToolFEModelToolBar() ) );
	if ( m_pFEModelToolBar )
		m_pToolPopup->setItemChecked( id, m_pFEModelToolBar->isVisible() );
} // slotToolPopupAboutToShow

// slot
void CMainWindow::slotImagePalettePopupAboutToShow()
{
	m_pImagePalettePopup->clear();

	m_pImagePalettePopup->insertTearOffHandle();

	int id[4];

	id[0] = m_pImagePalettePopup->insertItem( /* whiteToBlackColorIcon, */
		tr( "White to Black Color" ), this, SLOT( slotImagePaletteWhiteToBlackColor() ) );
	id[1] = m_pImagePalettePopup->insertItem( /* blackToWhiteColorIcon, */
		tr( "Black to White Color" ), this, SLOT( slotImagePaletteBlackToWhiteColor() ) );
	id[2] = m_pImagePalettePopup->insertItem( /* singleColorIcon, */
		tr( "Single Color" ), this, SLOT( slotImagePaletteSingleColor() ) );
	id[3] = m_pImagePalettePopup->insertItem( /* indexColorIcon, */
		tr( "Index Color" ), this, SLOT( slotImagePaletteIndexColor() ) );

	m_pImagePalettePopup->insertSeparator();

	QPixmap selectSingleColorIcon = QPixmap( selectSingleColor_xpm );
	m_pImagePalettePopup->insertItem( selectSingleColorIcon, tr( "Select Single Color" ), 
		this, SLOT( slotImagePaletteSelectSingleColor() ) );

	QPixmap selectIndexRangeIcon = QPixmap( selectIndexRange_xpm );
	m_pImagePalettePopup->insertItem( selectIndexRangeIcon, tr( "Select Index Range" ), 
		this, SLOT( slotImagePaletteSelectIndexRange() ) );

	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
		int ip = g->openglView()->imagePalette();

		m_pImagePalettePopup->setItemChecked( id[ip-1], true );
	}
} // slotImagePalettePopupAboutToShow

// slot
void CMainWindow::slotPreDefinedViewPopupAboutToShow()
{
	m_pPreDefinedViewPopup->clear();

	m_pPreDefinedViewPopup->insertTearOffHandle();

	m_pPreDefinedViewPopup->insertItem( tr ( "Top View" ),
		this, SLOT( slotPlusZ() ) );
	m_pPreDefinedViewPopup->insertItem( tr ( "Bottom View" ),
		this, SLOT( slotMinusZ() ) );
	m_pPreDefinedViewPopup->insertItem( tr ( "Left View" ),
		this, SLOT( slotPlusY() ) );
	m_pPreDefinedViewPopup->insertItem( tr ( "Right View" ),
		this, SLOT( slotMinusY() ) );
	m_pPreDefinedViewPopup->insertItem( tr ( "Front View" ),
		this, SLOT( slotPlusX() ) );
	m_pPreDefinedViewPopup->insertItem( tr ( "Back View" ),
		this, SLOT( slotMinusX() ) );
} // slotPreDefinedViewPopupAboutToShow

// slot
void CMainWindow::slot3DModelMenuAboutToShow()
{
	m_p3DModelMenu->clear();

	QWidget *widget;
	if ( ws )
		widget = ws->activeWindow();

	int id;

	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;

		id = m_p3DModelMenu->insertItem( tr( "Show Segmentation" ),
			this, SLOT( slotShowSegmentation() ) );
		m_p3DModelMenu->setItemChecked( id, m->planView()->isSegmentationShow() );
		if ( m->planView()->viewType() != 3 )
			m_p3DModelMenu->setItemEnabled( id, false );

		QPixmap segmentationIcon = QPixmap( segmentation_xpm );
		id = m_p3DModelMenu->insertItem( segmentationIcon, tr( "Segmentation..." ),
			this, SLOT( slotSegmentation() ), Key_F5 );	// Key_F5
		if ( m->planView()->viewType() != 3 )
			m_p3DModelMenu->setItemEnabled( id, false );

		m_p3DModelMenu->insertSeparator();

		m_p3DModelMenu->insertItem( tr( "FE-Model" ),
			this, SLOT( slotFEModel() ), Key_F7 );		// Key_F7

		m_p3DModelMenu->insertItem( tr( "Render Volume" ),
			this, SLOT( slotRenderVolume() ), Key_F8 );	// Key_F8

		m_p3DModelMenu->insertItem( tr( "Montage View" ),
			this, SLOT( slotMontageView() ), Key_F9 );	// Key_F9

		m_p3DModelMenu->insertItem( tr( "Surface Model" ),
			this, SLOT( slotMarchingCubeView() ), Key_F10 );	// key_10
	}
	else if ( widget && widget->isA( "GLMDIWindow" ) )
	{
	//	GLMDIWindow *g = (GLMDIWindow *) widget;

		m_p3DModelMenu->insertItem( tr( "Mesh Filtering..." ),
			this, SLOT( slotMeshFiltering() ) );
	}

	if ( widget && !widget->isA( "RenderMDIWindow" ) )
		m_p3DModelMenu->insertSeparator();

	if ( widget && widget->isA( "GLMDIWindow" ) )
		m_p3DModelMenu->insertItem( tr( "Volume Information..." ),
			this, SLOT( slotVolumeInfo() ) );

	m_p3DModelMenu->insertItem( tr( "Voxel Information..." ),
		this, SLOT( slotVoxelInfo() ) );

	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		id = m_p3DModelMenu->insertItem( tr( "Filter Information..." ),
				this, SLOT( slotMeshFilterInfo() ) );

		 if ( !((GLMDIWindow *)widget)->volume()->isMeshFiltered() )
			 m_p3DModelMenu->setItemEnabled( id, false );

		 m_p3DModelMenu->insertSeparator();

		 m_p3DModelMenu->insertItem( tr( "Analysis Volume..." ),
				this, SLOT( slotAnalysisVolume() ) );
	}
} // slot3DModelMenuAboutToShow

// slot
void CMainWindow::slotImageMenuAboutToShow()
{
	m_pImageMenu->clear();

	QWidget *widget = ws->activeWindow();

	if ( widget && widget->isA( "MDIWindow" ) )
	{
		m_pImageMenu->insertItem( tr( "Resampling..." ),
			this, SLOT( slotResampling() ) );
	}
	else if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		m_pImageMenu->insertItem( tr( "Save As Image..." ),
			this, SLOT( slotSaveAsImage() ) );
	}
	else if ( widget && widget->isA( "RenderMDIWindow" ) )
	{
		m_pImageMenu->insertItem( tr( "Save As Image..." ),
			this, SLOT( slotSaveAsImage() ) );

		m_pImageMenu->insertSeparator();

		m_pImageMenu->insertItem( tr( "Rendered Image Size..." ),
			this, SLOT( slotRenderedImageSize() ) );

		m_pImageMenu->insertItem( tr( "Render Info..." ),
			this, SLOT( slotRenderInfo() ) );
	}
} // slotImageMenuAboutToShow

// slot
void CMainWindow::slotHelpMenuAboutToShow()
{
	m_pHelpMenu->clear();

	m_pHelpMenu->insertItem( tr ( "About MGIBA..." ),
			this, SLOT( slotAboutMGIBA() ) );
} // slotHelpMenuAboutToShow

void CMainWindow::changeMenuAndToolBar( int type )
{
	if ( type == PLAN_VIEW )
	{
		menuBar()->setItemEnabled( m_n3DModelMenuId, true );
		menuBar()->setItemEnabled( m_nImageMenuId, true );

		m_pPlanViewToolBar->setEnabled( true );
		m_pViewToolBar->setEnabled( false );
		m_pFEModelToolBar->setEnabled( false );
	}
	else if ( type == OPENGL_VIEW )
	{
		menuBar()->setItemEnabled( m_n3DModelMenuId, true );
		menuBar()->setItemEnabled( m_nImageMenuId, true );

		m_pPlanViewToolBar->setEnabled( false );
		m_pViewToolBar->setEnabled( true );
			m_pZoomIn->setEnabled( false );
			m_pZoomOut->setEnabled( false );
		m_pFEModelToolBar->setEnabled( true );
	}
	else if ( type == RENDER_VIEW )
	{
		menuBar()->setItemEnabled( m_n3DModelMenuId, true );
		menuBar()->setItemEnabled( m_nImageMenuId, true );

		m_pPlanViewToolBar->setEnabled( false );
		m_pViewToolBar->setEnabled( true );
			m_pZoomIn->setEnabled( true );
			m_pZoomOut->setEnabled( true );
		m_pFEModelToolBar->setEnabled( false );
	}
	else
	{
		menuBar()->setItemEnabled( m_n3DModelMenuId, false );
		menuBar()->setItemEnabled( m_nImageMenuId, false );
	
		m_pPlanViewToolBar->setEnabled( false );
		m_pViewToolBar->setEnabled( false );
		m_pFEModelToolBar->setEnabled( false );
	}
} // changeMenuBar


/*************/
/* file menu */
/*************/

// slot
void CMainWindow::slotOpen()
{
	QString fn = QFileDialog::getOpenFileName( NULL, "Volume File (*.vol;*.VOL)", this );
    if ( !fn.isEmpty() )
	{
		// volume is already opened..!!
		if ( !checkVolumeList( fn ) )
			return;

		MDIWindow *m = (MDIWindow *) newDoc( PLAN_VIEW );
		if ( !m ) 
			return;

		if ( m->open( fn ) )
		{
			m_listVolume.append( fn );

			m->show();
			m->setIcon( QPixmap( planViewDocument_xpm ) );

			slot3DModelMenuAboutToShow();	// in order to activate accel key..

			// recent files
			checkInRecentFile( fn );
		}
		else
		{
			changeMenuAndToolBar( NONE_VIEW );
			if ( m ) 
			{
				delete m;
				m = NULL;
			}
		}
	}
} // slotOpen

// slot
void CMainWindow::slotSave()
{
	QWidget *widget = ws->activeWindow();

	if ( !widget )
		return;

	QString fn = QFileDialog::getSaveFileName( NULL, "Volume File (*.vol)", this );
    if ( !fn.isEmpty() )
	{
		// check file extension
		if ( fn.find( ".vol", 0 ) != -1 || fn.find( ".VOL", 0 ) != -1 )
			;
		else
			fn.append( ".vol" );
	}

	MDIWindow *m;

	if ( widget->isA( "MDIWindow" ) )
		m = (MDIWindow *) widget;
	else if ( widget->isA( "GLMDIWindow" ) )
		m = ((GLMDIWindow *) widget)->mdiWindow();
	else if ( widget->isA( "RenderMDIWindow" ) )
		m = ((RenderMDIWindow *) widget)->mdiWindow();
	else if ( widget->isA( "MontageMDIWindow" ) )
		m = ((MontageMDIWindow *) widget)->mdiWindow();

	if ( !m ) return;

	m->save( fn );
	
	// recent files
	checkInRecentFile( fn );
} // slotSave 

// slot
void CMainWindow::slotImportStack()
{
	CImportStackWizard wizard( this, "Import Stack Wizard", true );

	if ( !wizard.exec() == QDialog::Accepted )
		return;

	MDIWindow *m = (MDIWindow *) newDoc( PLAN_VIEW );
	if ( !m ) 
		return;

	if ( m->importStack( wizard.keyword() ) )
	{
		m->show();
		m->setCaption( "unnamed volume" );
		m->setIcon( QPixmap( planViewDocument_xpm ) );

		slot3DModelMenuAboutToShow();	// in order to activate accel key..
	}
	else
	{
		changeMenuAndToolBar( NONE_VIEW );
		if ( m )
		{
			delete m;
			m = NULL;
		}
	}
} // slotImportStack


/*************/
/* view menu */
/*************/

// slot
void CMainWindow::slotZoomIn()
{
	QWidget *widget = ws->activeWindow();

	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;
		m->planView()->slotZoomIn();
	}
	else if ( widget && widget->isA( "RenderMDIWindow" ) )
	{
		RenderMDIWindow *r = (RenderMDIWindow *) widget;
		r->zoomIn();
	}
} // slotZoomIn

// slot
void CMainWindow::slotZoomOut()
{
	QWidget *widget = ws->activeWindow();

	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;
		m->planView()->slotZoomOut();
	}
	else if ( widget && widget->isA( "RenderMDIWindow" ) )
	{
		RenderMDIWindow *r = (RenderMDIWindow *) widget;
		r->zoomOut();
	}
} // slotZoomOut

// slot
void CMainWindow::slotZoomAll()
{
	QWidget *widget = ws->activeWindow();
	// plan view
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *plan = (MDIWindow *) widget;
		plan->planView()->slotZoomAll();
	}
	// 3d model view
	else if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotZoomAll();
	}
} // slotZoomAll

// slot
void CMainWindow::slotDefaultView()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotZoomAll( true );
	}
} // slotDefaultView

// slot
void CMainWindow::slotXView()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *plan = (MDIWindow *) widget;
		plan->planView()->slotXView();
	}
} // slotXView

// slot
void CMainWindow::slotYView()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *plan = (MDIWindow *) widget;
		plan->planView()->slotYView();
	}
} // slotYView

// slot
void CMainWindow::slotZView()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *plan = (MDIWindow *) widget;
		plan->planView()->slotZView();
	}
} // slotZView

// slot
void CMainWindow::slotNextSlice()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *plan = (MDIWindow *) widget;
		plan->planView()->slotNextSlice();
	}
} // slotPreviousSlice
 
// slot
void CMainWindow::slotPreviousSlice()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *plan = (MDIWindow *) widget;
		plan->planView()->slotPreviousSlice();
	}
} // slotPreviousSlice

// slot
void CMainWindow::slotVoxelInfo()
{
	QWidget *widget = ws->activeWindow();
	if ( widget )
	{
		// if widget is OpenGL MDI window
		// you can change element size with modaless dialog
		if ( widget->isA( "GLMDIWindow" ) )
		{
			GLMDIWindow *g = (GLMDIWindow *) widget;
			if ( g->hasVoxelInfoDlg() )
				return;

			CVoxelInfoDlg *pDlg = new CVoxelInfoDlg( this, NULL, false );
			pDlg->init( widget );
			pDlg->exec();
		}
		else
		{
			CVoxelInfoDlg dlg( this, NULL, true );
			dlg.init( widget );
			dlg.exec();
		}
	}
} // slotVoxelInfo


/*****************/
/* 3D model menu */
/*****************/

// slot
void CMainWindow::slotShowSegmentation()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *plan = (MDIWindow *) widget;

		if ( plan->planView()->isSegmentationShow() )
		{
			plan->planView()->setShowSegmentation( false );
			m_pSegmentationShowHide->setOn( false );
		}
		else
		{
			statusBar()->message( tr( "Show Segmentation..." ) );

			plan->planView()->setShowSegmentation( true );
			m_pSegmentationShowHide->setOn( true );

			statusBar()->message( tr( "Ready" ) );
		}
	}
} // slotShowSegmentation 

// slot
void CMainWindow::slotSegmentation()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *plan = (MDIWindow *) widget;

		if ( !plan->hasSegmentationDlg() )
		{
			// set segmentation is true
			plan->planView()->setShowSegmentation( true );
			m_pSegmentationShowHide->setOn( true );

			// show segmentation dialog
			CSegmentationDlg *pDlg = new CSegmentationDlg( this, "Segmentation", false );
			pDlg->init( plan );
			pDlg->exec();
		}
	}
} // slotSegmentation

// slot
void CMainWindow::slotFEModel()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *plan = (MDIWindow *) widget;

		if ( plan->hasOpenGLView() )
		{
			QMessageBox::information( this, "Information",
				tr( "You already have a 3D Model." ) );
			return;
		}

		GLMDIWindow *opengl = (GLMDIWindow *) newDoc( OPENGL_VIEW );

		if ( !opengl->init( plan->volume() ) )
		{
			if ( opengl )
			{
				delete opengl;
				opengl = NULL;
			}

			return;
		}

		opengl->setIcon( QPixmap( openGLViewDocument_xpm ) );

		opengl->setMDIWindow( plan );
		plan->setOpenGLView( opengl );

		opengl->show();

		m_pFilteredShowHide->setOn( true );
		m_pTitleShowHide->setOn( true );
		m_pBoundingBoxShowHide->setOn( true );
		m_pGridLineShowHide->setOn( true );
	}
} // slotFEModel

// slot
void CMainWindow::slotRenderVolume()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *plan = (MDIWindow *) widget;

		if ( plan->hasRenderView() )
		{
			QMessageBox::information( this, "Information",
				tr( "You already have a Render View." ) );
			return;
		}

		RenderMDIWindow *render = (RenderMDIWindow *) newDoc( RENDER_VIEW );

		if ( !render->init( plan->volume() ) )
		{
			if ( render )
			{
				delete render;
				render = NULL;
			}

			return;
		}

		render->setIcon( QPixmap( renderViewDocument_xpm ) );

		render->setMDIWindow( plan );
		plan->setRenderView( render );

		render->show();
	}
} // slotRenderVolume

void CMainWindow::slotMontageView()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *plan = (MDIWindow *) widget;

		if ( plan->hasMontageView() )
		{
			QMessageBox::information( this, "Information",
				tr( "You already have a Montage View." ) );
			return;
		}

		MontageMDIWindow *montage = (MontageMDIWindow *) newDoc( MONTAGE_VIEW );

		if ( !montage->init( plan->volume(), plan->currentSlice() ) )
			return;

		montage->setIcon( QPixmap( montageViewDocument_xpm ) );

		montage->setMDIWindow( plan );
		plan->setMontageView( montage );

		montage->show();
	}
} // slotMontageView

void CMainWindow::slotMarchingCubeView()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *plan = (MDIWindow *) widget;

		if ( plan->hasMarchingCubeView() )
		{
			QMessageBox::information( this, "Information",
				tr( "You already have a Surface Model." ) );
			return;
		}

		MarchingCubeMDIWindow *mcube = (MarchingCubeMDIWindow *) newDoc( MCUBE_VIEW );

		if ( !mcube->init( plan->volume() ) )
			return;

		mcube->setMDIWindow( plan );
		plan->setMarchingCubeView( mcube );

		mcube->show();
	}
} // slotMarchingCubeView

// slot
void CMainWindow::slotMeshFiltering()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
	
		g->slotMeshFiltering();
	}
} // slotMeshFiltering

// slot
void CMainWindow::slotMeshFilterInfo()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;

		g->slotMeshFilterInfo();
	}
} // slotMeshFilterInfo

// slot
void CMainWindow::slotAnalysisVolume()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;

		g->slotAnalysisVolume();
	}
} // slotAnalysisVolume

// slot
void CMainWindow::slotVolumeInfo()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
	
		CVolumeInfoDlg dlg( this, NULL, true );
		if ( dlg.init( g->volume() ) )
			dlg.exec();
	}
} // slotVolumeInfo

void CMainWindow::slotRenderedImageSize()
{
	QWidget *m = ws->activeWindow();
	if ( m && m->isA( "RenderMDIWindow" ) )
	{
		RenderMDIWindow *r = (RenderMDIWindow *) m;

		r->slotRenderedImageSize();
	}
} // slotRenderedImageSize

void CMainWindow::slotRenderInfo()
{
	QWidget *m = ws->activeWindow();
	if ( m && m->isA( "RenderMDIWindow" ) )
	{
		RenderMDIWindow *r = (RenderMDIWindow *) m;

		r->slotRenderInfo();
	}
} // slotRenderInfo


/**************/
/* image menu */
/**************/

// slot
void CMainWindow::slotResampling()
{
	QWidget *widget = ws->activeWindow();

	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;
		m->resampling();
	}
} // slotResampling

// slot
void CMainWindow::slotSaveAsImage()
{

	QWidget *widget = ws->activeWindow();

	if ( widget && widget->isA( "RenderMDIWindow" ) )
	{
		RenderMDIWindow *r = (RenderMDIWindow *) widget;
		r->slotSaveAsImage();
	}
	else if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->slotSaveAsImage();
	}
} // slotSaveAsImage

 
/*************/
/* help menu */
/*************/

// slot
void CMainWindow::slotAboutMGIBA()
{
	CAboutDlg *pDlg = new CAboutDlg( this, NULL, true );
	pDlg->show();
} // slotAboutMGIBA


/*******************/
/* measure toolbar */
/*******************/

// slot
void CMainWindow::slotROI()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;

		if ( m_pROI->isOn() )
		{
			m_pROI->setOn( true );
			m->planView()->enterDrawMode( /*ROI_FREE_REGION*/ );

			statusBar()->message(
				tr( "Region Of Interest : Press any key to complete ROI..." ) );
		}
		else
		{
			m_pROI->setOn( false );
			m->planView()->leaveDrawMode();

			statusBar()->message( tr( "Ready" ) );
		}
	}
	else
		m_pROI->setOn( false );

	m_pMeasureLength->setOn( false );
	m_pMeasureAngle->setOn( false );
//	m_pMeasureRegion->setOn( false );
} // slotROI

// slot
void CMainWindow::slotROIClearROI()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;
		m->planView()->clearROI();

		statusBar()->message( tr( "Ready" ) );
	}
} // slotROIClearROI

// slot
void CMainWindow::slotMeasureLength()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;

		if ( m_pMeasureLength->isOn() )
		{
			m_pMeasureLength->setOn( true );
			m->planView()->enterDrawMode( /*MEASURE_MEASURE*/ );

			m->planView()->setShowMeasureLine( true );
			m_pMeasureLineShowHide->setOn( true );
		}
		else
		{
			m_pMeasureLength->setOn( false );
			m->planView()->leaveDrawMode();
		}
	}
	else
		m_pMeasureLength->setOn( false );

	m_pMeasureAngle->setOn( false );
//	m_pMeasureRegion->setOn( false );
	m_pROI->setOn( false );

	m_pFillSegmentation->setOn( false );
	m_pEraseSegmentation->setOn( false );
} // slotMeasureLength

// slot
void CMainWindow::slotMeasureAngle()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;

		if ( m_pMeasureAngle->isOn() )
		{
			m_pMeasureAngle->setOn( true );
			m->planView()->enterDrawMode( /*MEASURE_ANGLE*/ );

			m->planView()->setShowMeasureLine( true );
			m_pMeasureLineShowHide->setOn( true );
		}
		else
		{
			m_pMeasureAngle->setOn( false );
			m->planView()->leaveDrawMode();
		}
	}
	else
		m_pMeasureAngle->setOn( false );

	m_pMeasureLength->setOn( false );
//	m_pMeasureRegion->setOn( false );
	m_pROI->setOn( false );

	m_pFillSegmentation->setOn( false );
	m_pEraseSegmentation->setOn( false );
} // slotMeasureAngle

/*
// slot
void CMainWindow::slotMeasureRegion()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;

		if ( m_pMeasureRegion->isOn() )
		{
			m_pMeasureRegion->setOn( true );

			m->planView()->setShowMeasureLine( true );
			m_pMeasureLineShowHide->setOn( true );
		}
	}
	else
		m_pMeasureRegion->setOn( false );

	m_pMeasureLength->setOn( false );
	m_pMeasureAngle->setOn( false );
	m_pROI->setOn( false );

	m_pFillSegmentation->setOn( false );
	m_pEraseSegmentation->setOn( false );
} // slotMeasureRegion
*/

// slot
void CMainWindow::slotShowMeasureLine()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *plan = (MDIWindow *) widget;

		if ( plan->planView()->isMeasureLineShow() )
		{
			plan->planView()->setShowMeasureLine( false );
			m_pMeasureLineShowHide->setOn( false );
		}
		else
		{
			plan->planView()->setShowMeasureLine( true );
			m_pMeasureLineShowHide->setOn( true );
		}
	}
} // slotShowMeasureLine

// slot
void CMainWindow::slotMeasureClearLine()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;
		m->planView()->clearMeasureLine();

		m->planView()->setShowMeasureLine( false );
		m_pMeasureLineShowHide->setOn( false );

		statusBar()->message( tr( "Ready" ) );
	}
} // slotMeasureClearLine

void CMainWindow::slotFillSegmentation()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;

		if ( m_pFillSegmentation->isOn() )
		{
			m_pFillSegmentation->setOn( true );
			m_pEraseSegmentation->setOn( false );

			m_pSegmentationShowHide->setOn( true );
			m->planView()->setShowSegmentation( true );
		}
		else
			m_pFillSegmentation->setOn( false );
	}
	else
		m_pFillSegmentation->setOn( false );
} // slotFillSegmentation

void CMainWindow::slotEraseSegmentation()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;

		if ( m_pEraseSegmentation->isOn() )
		{
			m_pEraseSegmentation->setOn( true );
			m_pFillSegmentation->setOn( false );

			m_pSegmentationShowHide->setOn( true );
			m->planView()->setShowSegmentation( true );
		}
		else
			m_pEraseSegmentation->setOn( false );
	}
	else
		m_pEraseSegmentation->setOn( false );
} // slotEraseSegmentation

void CMainWindow::slotUpdate3DModel()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;

		if ( m->hasOpenGLView() )
		{
			GLMDIWindow *g = m->openGLMDIWindow();
			g->resetSegmentation();
			g->openglView()->updateView();
		}

		if ( m->hasRenderView() )
		{
		}

		if ( m->hasMontageView() )
		{
		}
	}
} // slotUpdate3DModel


/****************/
/* view toolbar */
/****************/

// slot
void CMainWindow::slotPlusX()
{
	QWidget *widget = ws->activeWindow();

	if ( widget && widget->isA( "RenderMDIWindow" ) )
	{
		RenderMDIWindow *r = (RenderMDIWindow *) widget;
		r->plusX();
	}
	else if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotFrontView();
	}
} // slotPlusX

// slot
void CMainWindow::slotMinusX()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "RenderMDIWindow" ) )
	{
		RenderMDIWindow *r = (RenderMDIWindow *) widget;
		r->minusX();
	}
	else if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotBackView();
	}
} // slotMinusX

// slot
void CMainWindow::slotPlusY()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "RenderMDIWindow" ) )
	{
		RenderMDIWindow *r = (RenderMDIWindow *) widget;
		r->plusY();
	}
	else if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotLeftView();
	}
} // slotPlusY

// slot
void CMainWindow::slotMinusY()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "RenderMDIWindow" ) )
	{
		RenderMDIWindow *r = (RenderMDIWindow *) widget;
		r->minusY();
	}
	else if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotRightView();
	}
} // slotMinusY

// slot
void CMainWindow::slotPlusZ()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "RenderMDIWindow" ) )
	{
		RenderMDIWindow *r = (RenderMDIWindow *) widget;
		r->plusZ();
	}
	else if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotTopView();
	}
} // slotPlusZ

// slot
void CMainWindow::slotMinusZ()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "RenderMDIWindow" ) )
	{
		RenderMDIWindow *r = (RenderMDIWindow *) widget;
		r->minusZ();
	}
	else if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotBottomView();
	}
} // slotMinusZ


/****************/
/* export popup */
/****************/

// slot
void CMainWindow::slotExportToAnsys()
{
	QWidget *widget = ws->activeWindow();
	if ( !widget )
		return;

	if ( widget && widget->isA( "MDIWindow" ) )
	{
		MDIWindow *m = (MDIWindow *) widget;
		m->exportToAnsys();
	}
	else if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->mdiWindow()->exportToAnsys();
	}
} // slotExportToAnsys

// slot
void CMainWindow::slotExportToSTLAscii()
{
	QWidget *w = ws->activeWindow();
	if ( !w )
		return;

	if ( !( w->isA( "MDIWindow" ) ||
		    w->isA( "GLMDIWindow" ) ||
		    w->isA( "MarchingCubeMDIWindow" ) ) )
		return;

	QString fn = QFileDialog::getSaveFileName( NULL, "STL File (*.stl)", this );
	if ( !fn.isEmpty() )
	{
		// check file extension
		if ( fn.find( ".stl", 0 ) != -1 || fn.find( ".STL", 0 ) != -1 )
			;
		else
			fn.append( ".stl" );

		if ( w->isA( "MDIWindow" ) )
			((MDIWindow *)w)->exportToSurface( fn, SRF_STL_ASCII );
		else if ( w->isA( "GLMDIWindow" ) )
			((GLMDIWindow *)w)->mdiWindow()->exportToSurface( fn, SRF_STL_ASCII );
		else if ( w->isA( "MarchingCubeMDIWindow" ) )
			((MarchingCubeMDIWindow *)w)->exportToSTL( fn );
	}
} // slotExportToSTL

// slot
void CMainWindow::slotExportToAcadScript()
{
	MDIWindow *m = (MDIWindow *) ws->activeWindow();
	if ( !m )
		return;

	QString fn = QFileDialog::getSaveFileName( NULL, "Auto Script File (*.scr)", this );
	if ( !fn.isEmpty() )
	{
		// check file extension
		if ( fn.find( ".scr", 0 ) != -1 || fn.find( ".SCR", 0 ) != -1 )
			;
		else
			fn.append( ".scr" );

		m->exportToSurface( fn, SRF_ACAD_SCRIPT );
	}
} // slotExportToAcadScript

// slot
void CMainWindow::slotExportToAcadDxf()
{
	MDIWindow *m = (MDIWindow *) ws->activeWindow();
	if ( !m )
		return;

	QString fn = QFileDialog::getSaveFileName( NULL, "DXF File (*.dxf)", this );
	if ( !fn.isEmpty() )
	{
		// check file extension
		if ( fn.find( ".dxf", 0 ) != -1 || fn.find( ".DXF", 0 ) != -1 )
			;
		else
			fn.append( ".dxf" );

		m->exportToSurface( fn, SRF_ACAD_DXF );
	}
} // slotExportToAcadDxf


/**************/
/* tool popup */
/**************/

// slot
void CMainWindow::slotToolMainToolBar()
{
	if ( m_pMainToolBar->isVisible() )
		m_pMainToolBar->hide();
	else
		m_pMainToolBar->show();
} // slotToolMainToolBar

// slot
void CMainWindow::slotToolSegMeasureToolBar()
{
	if ( m_pPlanViewToolBar->isVisible() )
		m_pPlanViewToolBar->hide();
	else
		m_pPlanViewToolBar->show();
} // slotToolSegMeasureToolBar

// slot
void CMainWindow::slotToolViewToolBar()
{
	if ( m_pViewToolBar->isVisible() )
		m_pViewToolBar->hide();
	else
		m_pViewToolBar->show();
} // slotToolViewToolBar

// slot
void CMainWindow::slotToolFEModelToolBar()
{
	if ( m_pFEModelToolBar->isVisible() )
		m_pFEModelToolBar->hide();
	else
		m_pFEModelToolBar->show();
} // slotToolFEModelToolBar


/***********************/
/* image palette popup */
/***********************/

// slot
void CMainWindow::slotImagePaletteWhiteToBlackColor()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		if ( !m_pWhiteToBlackColor->isOn() )
		{
			m_pWhiteToBlackColor->setOn( true );
			return;
		}

		m_pWhiteToBlackColor->setOn( true );
		m_pBlackToWhiteColor->setOn( false );
		m_pSingleColor->setOn( false );
		m_pIndexColor->setOn( false );

		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotWhiteToBlackColor();
	}
} // slotImagePaletteWhiteToBlackColor

// slot
void CMainWindow::slotImagePaletteBlackToWhiteColor()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		if ( !m_pBlackToWhiteColor->isOn() )
		{
			m_pBlackToWhiteColor->setOn( true );
			return;
		}

		m_pWhiteToBlackColor->setOn( false );
		m_pBlackToWhiteColor->setOn( true );
		m_pSingleColor->setOn( false );
		m_pIndexColor->setOn( false );

		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotBlackToWhiteColor();
	}
} // slotImagePaletteBlackToWhiteColor

// slot
void CMainWindow::slotImagePaletteSingleColor()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		if ( !m_pSingleColor->isOn() )
		{
			m_pSingleColor->setOn( true );
			return;
		}

		m_pWhiteToBlackColor->setOn( false );
		m_pBlackToWhiteColor->setOn( false );
		m_pSingleColor->setOn( true );
		m_pIndexColor->setOn( false );

		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotSingleColor();
	}
} // slotImagePaletteSingleColor

// slot
void CMainWindow::slotImagePaletteIndexColor()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		if ( !m_pIndexColor->isOn() )
		{
			m_pIndexColor->setOn( true );
			return;
		}

		m_pWhiteToBlackColor->setOn( false );
		m_pBlackToWhiteColor->setOn( false );
		m_pSingleColor->setOn( false );
		m_pIndexColor->setOn( true );

		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotIndexColor();
	}
} // slotImagePaletteIndexColor

// slot
void CMainWindow::slotImagePaletteSelectSingleColor()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotSelectSingleColor();
	}
} // slotImagePaletteSelectSingleColor

// slot
void CMainWindow::slotImagePaletteSelectIndexRange()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;
		g->openglView()->slotSelectIndexRange();
	}
} // slotImagePaletteSelectIndexRange

void CMainWindow::slotFilteredShowHide()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;

		if ( g->openglView()->isFilteredShow() )	// if show
		{
			g->openglView()->setShowFilteredVoxel( false );
			m_pFilteredShowHide->setOn( false );
		}
		else
		{
			g->openglView()->setShowFilteredVoxel( true );
			m_pFilteredShowHide->setOn( true );
		}

	}
} // slotFilteredShowHide

void CMainWindow::slotTitleShowHide()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;

		if ( g->openglView()->isTitleShow() )	// if show
		{
			g->openglView()->setShowTitle( false );
			m_pTitleShowHide->setOn( false );
		}
		else
		{
			g->openglView()->setShowTitle( true );
			m_pTitleShowHide->setOn( true );
		}
	}
} // slotTitleShowHide

void CMainWindow::slotBoundingBoxShowHide()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;

		if ( g->openglView()->isBoundingBoxShow() )	// if show
		{
			g->openglView()->setShowBoundingBox( false );
			m_pBoundingBoxShowHide->setOn( false );
		}
		else
		{
			g->openglView()->setShowBoundingBox( true );
			m_pBoundingBoxShowHide->setOn( true );
		}

	}
} // slotBoundingBoxShowHide

void CMainWindow::slotGridLineShowHide()
{
	QWidget *widget = ws->activeWindow();
	if ( widget && widget->isA( "GLMDIWindow" ) )
	{
		GLMDIWindow *g = (GLMDIWindow *) widget;

		if ( g->openglView()->isGridLineShow() )	// if show
		{
			g->openglView()->setShowGridLine( false );
			m_pGridLineShowHide->setOn( false );
		}
		else
		{
			g->openglView()->setShowGridLine( true );
			m_pGridLineShowHide->setOn( true );
		}

	}
} // slotGridLineShowHide



/****************/
/* recent files */
/****************/

// slot
void CMainWindow::slotRecentFile1()
{
	recentFileOpen( m_strRecentFiles[0] );
} // slotRecentFile1

// slot
void CMainWindow::slotRecentFile2()
{
	recentFileOpen( m_strRecentFiles[1] );
} // slotRecentFile2

// slot
void CMainWindow::slotRecentFile3()
{
	recentFileOpen( m_strRecentFiles[2] );
} // slotRecentFile3

// slot
void CMainWindow::slotRecentFile4()
{
	recentFileOpen( m_strRecentFiles[3] );
} // slotRecentFile4


/***********/
/* private */
/***********/

// private
void CMainWindow::recentFileOpen( QString fn )
{
	// volume is already opened..!!
	if ( !checkVolumeList( fn ) )
		return;

	MDIWindow *m = (MDIWindow *) newDoc( PLAN_VIEW );
	if ( !m ) 
		return;

	if ( m->open( fn ) )
	{
		m_listVolume.append( fn );

		m->show();
		m->setIcon( QPixmap( planViewDocument_xpm ) );

		slot3DModelMenuAboutToShow();	// in order to activate accel key..
	}
	else
	{
		changeMenuAndToolBar( NONE_VIEW );
		if ( m )
		{
			delete m;
			m = NULL;
		}
	}
} // recentFileOpen

// private
void CMainWindow::checkInRecentFile( QString fn )
{
	// check if "fn" is already in the recent files
	for ( int i=0 ; i<MAX_RECENT_FILES ; i++ )
	{
		if ( m_strRecentFiles[i] == fn )
			return;
	}

	// there is no "fn" in the recent files
	if ( m_nRecentFiles == MAX_RECENT_FILES )
	{
		for ( int i=0 ; i<MAX_RECENT_FILES ; i++ )
			m_strRecentFiles[i] = m_strRecentFiles[i+1];

		m_strRecentFiles[MAX_RECENT_FILES-1] = fn;
	}
	else
		m_strRecentFiles[m_nRecentFiles++] = fn;
} // checkInRecentFile

// public
void CMainWindow::checkOutRecentFile( QString fn )
{
	int index = 0;
	int i;
	for ( i=0 ; i<m_nRecentFiles ; i++ )
	{
		index++;

		if ( m_strRecentFiles[i] == fn )
			goto _NEXT_RET;
	}

_NEXT_RET:;

	int count = m_nRecentFiles - index;
	for ( i=index ; i<index+count ; i++ )
		m_strRecentFiles[i-1] = m_strRecentFiles[i];

	m_nRecentFiles--;
} // checkOutRecentFile

// private
bool CMainWindow::checkVolumeList( QString fn )
{
	for ( QStringList::Iterator it = m_listVolume.begin(); it != m_listVolume.end(); ++it )
	{
		if ( fn == *it )
		{
			QString str;
			str = tr ( "You already have [" ) + fn + tr ( "] in the workspace..!!" );

			QMessageBox::information( this, "Information", str );

			return false;
		}
	}

	return true;
} // checkVolumeList


/*******/
/* etc */
/*******/

// slot
void CMainWindow::slotWindowActivated( QWidget *w )
{
	if ( !w )
	{
		changeMenuAndToolBar( NONE_VIEW );
		return;
	}

	if ( !w->isA( "MDIWindow" ) )
	{
		m_pFillSegmentation->setOn( false );
		m_pEraseSegmentation->setOn( false );
	}

	if ( w->isA( "MDIWindow" ) )
	{
		changeMenuAndToolBar( PLAN_VIEW );

		MDIWindow *m = (MDIWindow *) w;
		/*
		if ( !plan->volume()->isOpen() )
			return;
		*/

		// check show segmentation
		if ( m->planView()->isSegmentationShow() )
		{
			m->planView()->setShowSegmentation( true );
			m_pSegmentationShowHide->setOn( true );
		}
		else
		{
			m->planView()->setShowSegmentation( false );
			m_pSegmentationShowHide->setOn( false );
		}
		// check show measure line
		if ( m->planView()->isMeasureLineShow() )
		{
			m->planView()->setShowMeasureLine( true );
			m_pMeasureLineShowHide->setOn( true );
		}
		else
		{
			m->planView()->setShowMeasureLine( false );
			m_pMeasureLineShowHide->setOn( false );
		}

		// check view type
		if ( m->planView()->viewType() != 3 )
		{
			m_pSegmentation->setEnabled( false );
			m_pSegmentationShowHide->setEnabled( false );
		}
		else
		{
			m_pSegmentation->setEnabled( true );
			m_pSegmentationShowHide->setEnabled( true );
		}

		/*
		// check tool bars
		if ( m_pMeasureLength->isOn() ||
			 m_pMeasureAngle->isOn() ||
			 m_pMeasureRegion->isOn() ||
			 m_pROI->isOn() )
			m->planView()->IDC_FRAME_PLANVIEW->setEnabled( false );
		else
			m->planView()->IDC_FRAME_PLANVIEW->setEnabled( true );
		*/
	}
	else if ( w->isA( "GLMDIWindow" ) )
	{
		changeMenuAndToolBar( OPENGL_VIEW );

		GLMDIWindow *g = (GLMDIWindow *) w;

		// check image palette
		m_pWhiteToBlackColor->setOn( false );
		m_pBlackToWhiteColor->setOn( false );
		m_pSingleColor->setOn( false );
		m_pIndexColor->setOn( false );

		int ip = g->openglView()->imagePalette();
		if ( ip == WHITE_BLACK_COLOR )
			m_pWhiteToBlackColor->setOn( true );
		else if ( ip == BLACK_WHITE_COLOR )
			m_pBlackToWhiteColor->setOn( true );
		else if ( ip == SINGLE_COLOR )
			m_pSingleColor->setOn( true );
		else if ( ip == INDEX_COLOR )
			m_pIndexColor->setOn( true );

		// check filtered voxel
		if ( g->openglView()->isFilteredShow() )
			m_pFilteredShowHide->setOn( true );
		else
			m_pFilteredShowHide->setOn( false );

		// check title, bounding box, grid line
		if ( g->openglView()->isTitleShow() )
			m_pTitleShowHide->setOn( true );
		else
			m_pTitleShowHide->setOn( false );

		if ( g->openglView()->isBoundingBoxShow() )
			m_pBoundingBoxShowHide->setOn( true );
		else
			m_pBoundingBoxShowHide->setOn( false );

		if ( g->openglView()->isGridLineShow() )
			m_pGridLineShowHide->setOn( true );
		else
			m_pGridLineShowHide->setOn( false );
	}
	else if ( w->isA( "RenderMDIWindow" ) )
		changeMenuAndToolBar( RENDER_VIEW );
	else if ( w->isA( "MontageMDIWindow" ) )
		changeMenuAndToolBar( MONTAGE_VIEW );
	else if ( w->isA( "MarchingCubeMDIWindow" ) )
		changeMenuAndToolBar( MCUBE_VIEW );
	else
		changeMenuAndToolBar( NONE_VIEW );
} // slotWindowActivated

// virtual 
void CMainWindow::closeEvent( QCloseEvent * )
{
	switch ( QMessageBox::information( this, "Close MGIBA",
			tr( "Do you want to exit MGIBA ?" ),
			"OK", "Cancel", 0, 0, 1 ) )
	{
	case 0:		// OK
	{
		// save recent files
		if ( m_nRecentFiles != 0 )
		{
			QFile f1( QDir::homeDirPath() + RC_RECENT_FILES );
			if ( f1.open( IO_WriteOnly ) )
			{
				QTextStream ts1( &f1 );
				
				ts1 << "# " << PROGRAM_NAME << " Ver. " << PROGRAM_VERSION << endl;
				ts1 << "# Recent Files" << endl;
				ts1 << m_nRecentFiles << endl;

				for ( int i=0 ; i<m_nRecentFiles ; i++ )
					ts1 << m_strRecentFiles[i] << endl;

				f1.close();
				ts1.unsetDevice();
			}
		}

		// save window position
		QFile f2( QDir::homeDirPath() + RC_WINDOW_POSITION );
		if ( f2.open( IO_WriteOnly ) )
		{
			QPoint p = pos();       // store position
			QSize s = size();       // store size

			QTextStream ts2( &f2 );

			ts2 << "# " << PROGRAM_NAME << " Ver. " << PROGRAM_VERSION << endl;
			ts2 << "# Window Position" << endl;
			ts2 << p.x() << " " << p.y() << endl;
			ts2 << s.width() << " " << s.height() << endl;

			f2.close();
			ts2.unsetDevice();
		}

		// remove temperary fles
		QFile::remove( QDir::homeDirPath() + "/.mesh_filter.tmp" );
		qApp->quit();      
	}
		break;
	case 1:		// Cancel
		break;
	}
}
