#include "MDIWindow.h"
#include "GLMDIWindow.h"
#include "MontageMDIWindow.h"
#include "PlanView.h"
#include "GLView.h"
#include "MontageView.h"
#include "VolumeFile.h"
#include "MainWindow.h"

#ifdef Q_WS_WIN
# include "winuser.h"
#endif

#include <math.h>
#include <stdlib.h>

#include <qapplication.h>
#include <qscrollview.h>
#include <qscrollbar.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <qcombobox.h>
#include <qtimer.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <qlist.h>
#include <qpen.h>
#include <qpainter.h>
#include <qregion.h>
#include <qcursor.h>
#include <qprogressdialog.h>
#include <qfile.h>

#include "cursor_xpm.h"

#include <qmessagebox.h>	// debug


////////////////////////////////////////////////////////////////////////////
//
// CPlanView
//

CPlanView::CPlanView( QWidget *parent, const char *name, WFlags f )
	: CPlanView_base( parent, name, f )
{
	m_nType = 3;			// default is z-view
	m_pMDIWindow = NULL;
	m_pVolume = NULL;
	m_pAnimationTimer = new QTimer( this );
	m_fRatio = 1.0;
	m_nCurrentSlice = 1;
	m_bShowSegmentation = false;
	m_bShowMeasureLine = false;

	connect( IDC_BTN_ZOOM, SIGNAL( clicked() ),
		this, SLOT( slotZoomAll() ) );
	connect( IDC_BTN_ZOOMIN, SIGNAL( clicked() ),
		this, SLOT( slotZoomIn() ) );
	connect( IDC_BTN_ZOOMOUT, SIGNAL( clicked() ),
		this, SLOT( slotZoomOut() ) );

	connect( IDC_BTN_ANIMATION, SIGNAL( clicked() ),
		this, SLOT( slotAnimationButton() ) );
	connect( m_pAnimationTimer, SIGNAL( timeout() ),
		this, SLOT( slotAnimation() ) );

	connect( IDC_COMBO_TYPE, SIGNAL( activated( int ) ),
		this, SLOT( slotTypeChanged( int ) ) );

	connect( this, SIGNAL( viewResized() ), 
		this, SLOT( slotMoveImageToCenter() ) );

	init( (MDIWindow *) parent );
} // CPlanView

CPlanView::~CPlanView()
{
	if ( m_pAnimationTimer )
	{
		delete m_pAnimationTimer;
		m_pAnimationTimer = NULL;
	}

	if ( m_pPlanViewArea )
	{
		delete m_pPlanViewArea;
		m_pPlanViewArea = NULL;
	}

	if ( m_pScrollView )
	{
		delete m_pScrollView;
		m_pScrollView = NULL;
	}
} // ~CPlanView

MDIWindow *CPlanView::mdiWindow()
{
	if ( m_pMDIWindow )
		return m_pMDIWindow;
	else
		return NULL;
} // mdiWindow

CVolume *CPlanView::volume()
{
	if ( m_pVolume )
		return m_pVolume;
	else
		return NULL;
} // volume

CPlanViewArea *CPlanView::planViewArea()
{
	if ( m_pPlanViewArea )
		return m_pPlanViewArea;
	else
		return NULL;
} // planViewArea

// private
bool CPlanView::init( MDIWindow *pMDIWindow )
{
	// 0. parent - MDIWindow
	m_pMDIWindow = pMDIWindow;
	ASSERT( m_pMDIWindow );

	// 1. scroll view
	m_pScrollView = new QScrollView( this );
	ASSERT( m_pScrollView );

	// 2. scroll area
	m_pPlanViewArea = new CPlanViewArea( this );

	m_pScrollView->addChild( m_pPlanViewArea );
	m_pScrollView->setGeometry( 0, 0, width(), height()-20 );

	// 3. set bottom frame
	IDC_FRAME_PLANVIEW->setGeometry( 0, height()-20, width(), 20 );

	return true;
} // init

bool CPlanView::isTimerOn()
{
	if ( m_pAnimationTimer->isActive() )
		return true;
	else
		return false;
}

void CPlanView::setVolume( CVolume *pVolume )
{
	m_pVolume = pVolume;
	ASSERT( m_pVolume );

	// set geometry of virtual volume according to view type
	if ( m_nType == 1 )			// x-view
	{
		m_nWidth  = m_pVolume->volumeHeight();
		m_nHeight = m_pVolume->volumeDepth();
		m_nDepth  = m_pVolume->volumeWidth();
	}
	else if ( m_nType == 2 )	// y-view
	{
		m_nWidth  = m_pVolume->volumeWidth();
		m_nHeight = m_pVolume->volumeDepth();
		m_nDepth  = m_pVolume->volumeHeight();
	}
	else if ( m_nType == 3 )	// z-view
	{
		m_nWidth  = m_pVolume->volumeWidth();
		m_nHeight = m_pVolume->volumeHeight();
		m_nDepth  = m_pVolume->volumeDepth();
	}

	// set bottom frame
	IDC_SPIN_SLICE->setRange( 1, m_nDepth );
	IDC_SPIN_SLICE->setValue( 1 );

	DWORD index;
	int gray;

	QImage image( m_nWidth, m_nHeight, 32 );

	// set initial image from volume
	for ( WORD i=1 ; i<=m_nHeight ; i++ )
	{
		for ( WORD j=1 ; j<=m_nWidth ; j++ )
		{
			// index = j + (i-1)*m_nWidth;
			index = voxelIndex( j, i, 1 );
			gray = (int) m_pVolume->voxelColor( index );
			image.setPixel( j-1, i-1, qRgb( gray, gray, gray ) );
		}
	}
	m_pPlanViewArea->init( image, 1, m_nType );
	m_pPlanViewArea->repaint( false );

	// signal-slot
	// after assigning the volume, connect signal and slot
	connect( IDC_SPIN_SLICE, SIGNAL( valueChanged( int) ),
		this, SLOT( slotSliceChanged( int ) ) );
} // setVolume

// private
DWORD CPlanView::voxelIndex( WORD x, WORD y, int slice )
{
	int index;
	if ( m_nType == 1 )			// x-view
		index = y + (x-1)*m_nDepth + (m_nHeight-y)*(m_nWidth*m_nDepth-1) + (slice-1);
	else if ( m_nType == 2 )	// y-view
		index = x + (m_nHeight-y)*(m_nWidth*m_nDepth) + (slice-1)*m_nWidth;
	else if ( m_nType == 3 )	// z-view
		index = x + (m_nHeight-y)*m_nWidth + (slice-1)*(m_nWidth*m_nHeight);

	return index;
} // voxelIndex

// private
void CPlanView::viewTypeChanged( int type )
{
	// if current view type is not changed, just return
	if ( type == 0 && m_nType == 3 )
		return;
	if ( type == 1 && m_nType == 1 )
		return;
	if ( type == 2 && m_nType == 2 )
		return;

	// set current view type
	if ( type == 0 )
		m_nType = 3;
	else if ( type == 1 )
		m_nType = 1;
	else if ( type == 2 )
		m_nType = 2;

	// set geometry of virtual volume according to view type
	if ( m_nType == 1 )			// x-view
	{
		m_nWidth  = m_pVolume->volumeHeight();
		m_nHeight = m_pVolume->volumeDepth();
		m_nDepth  = m_pVolume->volumeWidth();
	}
	else if ( m_nType == 2 )	// y-view
	{
		m_nWidth  = m_pVolume->volumeWidth();
		m_nHeight = m_pVolume->volumeDepth();
		m_nDepth  = m_pVolume->volumeHeight();
	}
	else if ( m_nType == 3 )	// z-view
	{
		m_nWidth  = m_pVolume->volumeWidth();
		m_nHeight = m_pVolume->volumeHeight();
		m_nDepth  = m_pVolume->volumeDepth();
	}

	// set tool bar
	if ( m_nType != 3 )
	{
		g_pMainWindow->m_pSegmentation->setEnabled( false );
		g_pMainWindow->m_pSegmentationShowHide->setEnabled( false );

		g_pMainWindow->m_pROI->setEnabled( false );
		g_pMainWindow->m_pROIShowHide->setEnabled( false );
	}
	else
	{
		g_pMainWindow->m_pSegmentation->setEnabled( true );
		g_pMainWindow->m_pSegmentationShowHide->setEnabled( true );

		g_pMainWindow->m_pROI->setEnabled( true );
		g_pMainWindow->m_pROIShowHide->setEnabled( true );
	}
} // typeChanged

bool CPlanView::isSegmentationShow()
{
	if ( m_bShowSegmentation )
		return true;
	else
		return false;
} // isSegmentationShow

void CPlanView::setShowSegmentation( bool flag )
{
	if ( m_bShowSegmentation == flag )
		return;

	m_bShowSegmentation = flag;

	slotSliceChanged( m_nCurrentSlice );
} // setShowSegmentation

bool CPlanView::isMeasureLineShow()
{
	if ( m_bShowMeasureLine )
		return true;
	else
		return false;
} // isMeasureLineShow

void CPlanView::setShowMeasureLine( bool flag )
{
	m_bShowMeasureLine = flag;

	if ( m_pPlanViewArea )
		m_pPlanViewArea->repaint( false );
} // setShowMeasureLine

bool CPlanView::checkViewType( int type )
{
	if ( type == m_nType )
		return true;
	else
		return false;
} // checkViewType

void CPlanView::enterDrawMode( /*int mode*/ )
{
	/*
	g_pMainWindow->menuBar()->setItemEnabled( g_pMainWindow->m_nViewMenuId, false );
	IDC_FRAME_PLANVIEW->setEnabled( false );
	*/

	if ( m_pPlanViewArea )
	{
		clearFocus();
		m_pPlanViewArea->setFocus();

		QPixmap measure( measureCursor32_xpm );
		QCursor measureCursor( measure );
		setCursor( measureCursor );

		m_pPlanViewArea->enterDrawMode( /*mode*/ );

#ifdef Q_WS_WIN
		QRect rt;
		QPoint pt, pt1, pt2;
		rt = geometry();
		pt.setX( rt.left() );
		pt.setY( rt.top() );

		pt1 = mapToGlobal( pt );

		pt.setX( rt.left() + rt.width() );
		pt.setY( rt.top() + rt.height() - IDC_FRAME_PLANVIEW->height() );

		pt2 = mapToGlobal( pt );

		RECT rect;

		rect.left = pt1.x();
		rect.top = pt1.y();
		rect.right = pt2.x();
		rect.bottom = pt2.y();

		::ClipCursor( &rect );
#endif // Q_WS_WIN
	}
} // enterDrawMode

void CPlanView::leaveDrawMode()
{
	/*
	g_pMainWindow->menuBar()->setItemEnabled( g_pMainWindow->m_nViewMenuId, true );
	IDC_FRAME_PLANVIEW->setEnabled( true );
	*/

	if ( m_pPlanViewArea )
	{
		setCursor( ArrowCursor );

		m_pPlanViewArea->leaveDrawMode();

#ifdef Q_WS_WIN
		::ClipCursor( NULL );
#endif // Q_WS_WIN
	}
} // leaveDrawMode

void CPlanView::clearROI()
{
	if ( m_pPlanViewArea)
	{
		m_pPlanViewArea->clearROI();

		g_pMainWindow->m_pROI->setOn( false );

		leaveDrawMode();
	}
} // clearROI

void CPlanView::clearMeasureLine()
{
	if ( m_pPlanViewArea )
	{
		m_pPlanViewArea->clearMeasureLine();

		g_pMainWindow->m_pMeasureLength->setOn( false );
		g_pMainWindow->m_pMeasureAngle->setOn( false );
//		g_pMainWindow->m_pMeasureRegion->setOn( false );

		leaveDrawMode();
	}
} // clearMeasureLine

bool CPlanView::resampling( QString &fileType, QString &directory, QString &keyword,
		int viewType, WORD wStart, WORD wEnd )
{
	int oldViewType = m_nType;

	viewTypeChanged( viewType );

	DWORD index;
	int gray;
	QString strFileName;

	int numFiles = wEnd - wStart + 1;
	int count = 0;

#ifdef Q_WS_WIN
	QString str = directory + "\\" + keyword;
#else
	QString str = directory + "/" + keyword;
#endif // Q_WS_WIN

    QProgressDialog progress( "Resampling...", "Abort", numFiles,
                                this, "progress", true );
	progress.show();

	for ( WORD w=wStart ; w<=wEnd ; w++ )
	{
		QImage image( m_nWidth, m_nHeight, 8, 256 );
		for ( int c=0 ; c<256 ; c++ )
			image.setColor ( c, qRgb( c, c, c ) );

		strFileName.sprintf( "%s%04d.%s", str.latin1(), w, fileType.lower().latin1() );
		if ( QFile::exists( strFileName ) )
			QFile::remove( strFileName );

		progress.setProgress( count++ );
		progress.setLabelText( strFileName );
		qApp->processEvents();

		if ( progress.wasCancelled() )
		{
			/*
			for ( WORD ww=wStart ; ww<=wEnd ; ww++ )
			{
				strFileName.sprintf( "%s%04d.%s", str.latin1(), w, fileType.lower().latin1() );
				if ( QFile::exists( strFileName ) )
					QFile::remove( strFileName );
			}
			*/
			return false;
		}

		for ( WORD i=1 ; i<=m_nHeight ; i++ )
		{
			for ( WORD j=1 ; j<=m_nWidth ; j++ )
			{
				index = voxelIndex( j, i, w );
				gray = (int) m_pVolume->voxelColor( index );
				image.setPixel( j-1, i-1, gray );
			}
		}

		if ( !image.save( strFileName, fileType ) )
			return false;
	}

	// restore view type
	viewTypeChanged( oldViewType );

	progress.setProgress( numFiles );
	progress.hide();

	return true;
} // resampling

// slot
void CPlanView::slotSetSlice( int slice )
{
	IDC_SPIN_SLICE->setValue( slice );
} // slotSetSlice

// slot
void CPlanView::slotSliceChanged( int slice )
{
	m_nCurrentSlice = slice;

	WORD wMin = m_pVolume->minThreshold( slice );
	WORD wMax = m_pVolume->maxThreshold( slice );

	DWORD index;
	int gray;
	int color;

	QList<QRegion> listRgnROI = m_pPlanViewArea->listRgnROI();
	int nCount = listRgnROI.count();

	// set bounding rect for display speed
	QList<QRect> boundingRectROI;
	if ( nCount != 0 )
	{
		for ( int ii=0 ; ii<nCount ; ii++ )
		{
			/*
			QRect *rt = new QRect;
			rt = &listRgnROI.at( ii )->boundingRect();
			*/
			QRect *rt = new QRect( listRgnROI.at( ii )->boundingRect() );
			boundingRectROI.append( rt );
		}
	}

	QImage image( m_nWidth, m_nHeight, 32 );

	// set image from volume
	for ( WORD i=1 ; i<=m_nHeight ; i++ )
	{
		for ( WORD j=1 ; j<=m_nWidth ; j++ )
		{
			index = voxelIndex( j, i, slice );
			gray = (int) m_pVolume->voxelColor( index );
			if ( m_bShowSegmentation )
			{
				QPoint pt( j-1, i-1 );
				bool bContains = true;

				// check whether point is inside ROI or not
				if ( nCount != 0 )
				{
					bContains = false;

					for ( int ii=0 ; ii<nCount ; ii++ )
					{
						if ( boundingRectROI.at( ii )->contains( pt ) )
						{
							QRegion *pRgn = listRgnROI.at( ii );
						
							if ( pRgn->contains( pt ) )
							{
								bContains = true;
								goto _NEXT_RET;
							}
						}
					}
				}

_NEXT_RET:;

				if ( gray >= wMin && gray <= wMax )
				{
					if ( bContains && m_pVolume->isMaskOn( index ) )
					{
						gray += 100;
						if ( gray > 255 ) gray = 255;
						color = qRgb( 0, gray, 0 );	// green
					}
					else
						color = qRgb( gray, gray, gray );
				}
				else
				{
					if ( bContains && m_pVolume->isMaskOn( index ) )
					{
						gray += 100;
						if ( gray > 255 ) gray = 255;
						color = qRgb( 0, gray, 0 );	// green
					}
					else
						color = qRgb( gray, gray, gray );
				}
			}
			else
				color = qRgb( gray, gray, gray );

			image.setPixel( j-1, i-1, color );
		}
	}
	m_pPlanViewArea->init( image, slice, m_nType );
	m_pPlanViewArea->repaint( false ); 

	emit sliceChanged( slice );

	// change montage view
	if ( m_pMDIWindow->hasMontageView() )
		m_pMDIWindow->montageMDIWindow()->montageView()->slotSetSlice( slice );
} // slotSliceChanged

// slot
void CPlanView::slotZoomIn()
{
	if ( m_fRatio != 7.0 )
	{
	//	m_fRatio = m_fRatio + 0.25;
		m_fRatio += 1;

		slotSliceChanged( m_nCurrentSlice );
		m_pPlanViewArea->scaleImage( m_fRatio );
		m_pPlanViewArea->repaint( false );

		emit viewResized();
	}
} // slotZoomIn

// slot
void CPlanView::slotZoomOut()
{
	if ( m_fRatio != 1 )
	{
	//	m_fRatio = m_fRatio - 0.25;
		m_fRatio -= 1;

		slotSliceChanged( m_nCurrentSlice );
		m_pPlanViewArea->scaleImage( m_fRatio );
		m_pPlanViewArea->repaint( false );

		emit viewResized();
	}
} // slotZoomOut

// slot
void CPlanView::slotZoomAll()
{
	if ( m_fRatio != 1.0 )
	{
		m_fRatio = 1.0;

		slotSliceChanged( m_nCurrentSlice );
		m_pPlanViewArea->scaleImage( m_fRatio );
		m_pPlanViewArea->repaint( false );

		emit viewResized();
	}
} // slotZoomAll

// slot
void CPlanView::slotXView()
{ 
	if ( m_nType != 1 )
	{
		IDC_COMBO_TYPE->setCurrentItem( 1 );	// 1 is x
		slotTypeChanged( 1 );
	}
} // slotXView

// slot
void CPlanView::slotYView()
{
	if ( m_nType != 2 )
	{
		IDC_COMBO_TYPE->setCurrentItem( 2 );	// 2 is y
		slotTypeChanged( 2 );
	}
} // slotYView

// slot
void CPlanView::slotZView()
{
	if ( m_nType != 3 )
	{
		IDC_COMBO_TYPE->setCurrentItem( 0 );	// 0 is z
		slotTypeChanged( 0 );
	}
} // slotZView

// slot
void CPlanView::slotTypeChanged( int type )
{
	viewTypeChanged( type );

	// set bottom frame
	disconnect( IDC_SPIN_SLICE, SIGNAL( valueChanged( int) ),
		this, SLOT( slotSliceChanged( int ) ) );
	
	IDC_SPIN_SLICE->setRange( 1, m_nDepth );
	IDC_SPIN_SLICE->setValue( 1 );

	connect( IDC_SPIN_SLICE, SIGNAL( valueChanged( int) ),
		this, SLOT( slotSliceChanged( int ) ) );

	slotSliceChanged( 1 );

	// clear all draw line entity
	m_pPlanViewArea->clearMeasureLine();
	m_pPlanViewArea->clearROI();

	emit viewResized();

	// change montage view
	if ( m_pMDIWindow->hasMontageView() )
		m_pMDIWindow->montageMDIWindow()->montageView()->slotTypeChanged( m_nType );
} // slotChangeType

// slot
void CPlanView::slotMoveImageToCenter()
{
	QScrollBar *vb = m_pScrollView->verticalScrollBar();
	QScrollBar *hb = m_pScrollView->horizontalScrollBar();

	int x = 0;
	int y = 0;
	int w = m_pScrollView->width();
	int h = m_pScrollView->height();

	if ( !hb->isVisible() && m_pPlanViewArea->width() < w )
		x = (int) ( ( w - m_pPlanViewArea->width() ) / 2 ) - 1;

	if ( !vb->isVisible() && m_pPlanViewArea->height() < h )
		y = (int) ( ( h - m_pPlanViewArea->height() ) / 2 ) - 1;

	m_pScrollView->moveChild( m_pPlanViewArea, x, y );
} // slotMoveImageToCenter

// slot
void CPlanView::slotNextSlice()
{
	if ( m_nCurrentSlice == m_nDepth )
		m_nCurrentSlice = 1;
	else
		m_nCurrentSlice++;

	IDC_SPIN_SLICE->setValue( m_nCurrentSlice );
} // slotNextSlice

// slot
void CPlanView::slotPreviousSlice()
{
	if ( m_nCurrentSlice == 1 )
		m_nCurrentSlice = m_nDepth;
	else
		m_nCurrentSlice--;

	IDC_SPIN_SLICE->setValue( m_nCurrentSlice );
} // slotPreviousSlice

// slot
void CPlanView::slotAnimationButton()
{
	if ( m_nDepth == 1 )
		return;

	if ( m_pAnimationTimer->isActive() )
		m_pAnimationTimer->stop();
	else
	{
		if ( m_pPlanViewArea->listRgnROI().count() == 0 )
			m_pAnimationTimer->start( 500 );
		else
			m_pAnimationTimer->start( 10 );
	}
} // slotAnimationButton

// slot
void CPlanView::slotAnimation()
{
	if ( m_pAnimationTimer->isActive() )
	{
		int slice = m_nCurrentSlice;

		if ( m_nCurrentSlice == m_nDepth )
			slice = 1;
		else
			slice++;

		IDC_SPIN_SLICE->setValue( slice );
	}
} // slotAnimation

// virtual
void CPlanView::resizeEvent( QResizeEvent * /*e*/ )
{
	m_pScrollView->setGeometry( 0, 0, width(), height()-20 );
	IDC_FRAME_PLANVIEW->setGeometry( 0, height()-20, width(), 20 );

	emit viewResized();
} // resizeEvent


////////////////////////////////////////////////////////////////////////////
//
// CPlanViewArea
//

CPlanViewArea::CPlanViewArea( QWidget *parent, const char *name, WFlags f )
	: QWidget( parent, name, f )
{
	m_pPlanView = (CPlanView *) parent;
	ASSERT( m_pPlanView );

	m_fRatio = 1.0;

	m_nPtCount = 0;
	m_bMeasure = false;

	m_listPtROI.setAutoDelete( true );

	m_bFillEraseSegmentation = false;

	m_pFillEraseSizePopup = NULL;
	m_nFillEraseSize = 1;		// 1, 2, 4, 8, 16 : default is 1

	setPopupMenu();

	setBackgroundMode( NoBackground );

	setMouseTracking( true );				// enable mouseMoveEvent
	setFocusPolicy( QWidget::StrongFocus );	// enable keyPressEvent
} // CPlanViewArea

CPlanViewArea::~CPlanViewArea()
{
	m_listPtMeasure.clear();
	m_listPtAngle.clear();

	m_listLineValue.clear();
	m_listAreaValue.clear();
	m_listAngleValue.clear();

	// ROI
	m_listPtROI.clear();	// for display
	m_listRgnROI.clear();

	if ( m_pFillEraseSizePopup )
	{
		delete m_pFillEraseSizePopup;
		m_pFillEraseSizePopup = NULL;
	}
} // ~CPlanViewArea

void CPlanViewArea::init( QImage image, int slice, int type )
{
	m_OriginalImage = image;
	m_nWidth = image.width();
	m_nHeight = image.height();

	m_nSlice = slice;
	m_nType = type;

	scaleImage( m_fRatio );
} // init

void CPlanViewArea::setPopupMenu()
{
	// fill erase popup
	m_pFillEraseSizePopup = new QPopupMenu;
	m_nFillEraseSizeId = 0;

	connect( m_pFillEraseSizePopup, SIGNAL( aboutToShow() ),
		this, SLOT( slotFillEraseSizePopupAboutToShow() ) );
} // setPopupMenu

void CPlanViewArea::slotFillEraseSizePopupAboutToShow()
{
	m_pFillEraseSizePopup->clear();

	m_pFillEraseSizePopup->insertItem( tr( "Fill/Erase Segmentation" ), -1 );
	m_pFillEraseSizePopup->insertSeparator();

//	QPixmap pixel1Icon = QPixmap( /*pixelWidth1_xpm*/ );
	m_pFillEraseSizePopup->insertItem( /*pixel1Icon,*/ tr( " 1 Pixel Width" ), 0 );

//	QPixmap pixel2Icon = QPixmap( /*pixelWidth2_xpm*/ );
	m_pFillEraseSizePopup->insertItem( /*pixel2Icon,*/ tr( " 2 Pixel Width" ), 1 );

//	QPixmap pixel4Icon = QPixmap( /*pixelWidth4_xpm*/ );
	m_pFillEraseSizePopup->insertItem( /*pixel4Icon,*/ tr( " 4 Pixel Width" ), 2 );

//	QPixmap pixel8Icon = QPixmap( /*pixelWidth8_xpm*/ );
	m_pFillEraseSizePopup->insertItem( /*pixel8Icon,*/ tr( " 8 Pixel Width" ), 3 );

//	QPixmap pixel16Icon = QPixmap( /*pixelWidth16_xpm*/ );
//	m_pFillEraseSizePopup->insertItem( /*pixel16Icon,*/ tr( "16 Pixel Width" ), 4 );

	m_pFillEraseSizePopup->setItemChecked( m_nFillEraseSizeId, true );


	/*
	if ( g_pMainWindow->m_pFillSegmentation->isOn() ||
		 g_pMainWindow->m_pEraseSegmentation->isOn() )
	*/
	{
		m_pFillEraseSizePopup->insertSeparator();

		m_pFillEraseSizePopup->insertItem( tr( "Fill Segmentation" ), 99 );
		m_pFillEraseSizePopup->insertItem( tr( "Erase Segmentation" ), 100 );

		QString str;
		if ( g_pMainWindow->m_pFillSegmentation->isOn() )
			str = tr( "Finish Filling" );
		else if ( g_pMainWindow->m_pEraseSegmentation->isOn() )
			str = tr( "Finish Erasing" );

		m_pFillEraseSizePopup->insertItem( str, 101 );

		if ( g_pMainWindow->m_pFillSegmentation->isOn() )
			m_pFillEraseSizePopup->setItemChecked( 99, true );
		else
			m_pFillEraseSizePopup->setItemChecked( 100, true );
	}
	
	// signal-slot
	connect( m_pFillEraseSizePopup, SIGNAL( activated(int) ),
		this, SLOT( slotFillEraseSize( int ) ) );
} // slotFillEraseSizePopupAboutToShow

void CPlanViewArea::slotFillEraseSize( int id )
{
	if ( id == -1 )
		return;

	if ( id == 99 )
	{
		g_pMainWindow->m_pFillSegmentation->setOn( true );
		g_pMainWindow->m_pEraseSegmentation->setOn( false );
	}
	else if ( id == 100 )
	{
		g_pMainWindow->m_pFillSegmentation->setOn( false );
		g_pMainWindow->m_pEraseSegmentation->setOn( true );
	}
	else if ( id == 101 )
	{
		g_pMainWindow->m_pFillSegmentation->setOn( false );
		g_pMainWindow->m_pEraseSegmentation->setOn( false );
		m_bFillEraseSegmentation = false;
	}
	else
	{
		m_nFillEraseSize = (int) pow( 2.0, id );
		m_nFillEraseSizeId = id;
	}
} // slotFillEraseSize

void CPlanViewArea::scaleImage( double fRatio )
{
	double fPreRatio = m_fRatio;
	// 1. rescale image
	m_fRatio = fRatio;

	int w = m_OriginalImage.width();
	int h = m_OriginalImage.height();

	int ww = (int) (w * m_fRatio);
	int hh = (int) (h * m_fRatio);

	m_Image = m_OriginalImage.smoothScale( ww, hh );

	m_szImage.setWidth( m_Image.width() );
	m_szImage.setHeight( m_Image.height() );

	// 2. resize plan view area
	resize( m_szImage );

	// 3. rescale point list, if it exists
	// 3-1. measure-measure
	if ( m_listPtMeasure.count() != 0 )
	{
		QPoint *pt;
		for ( pt=m_listPtMeasure.first() ; pt != 0 ; pt=m_listPtMeasure.next() )
		{
			if ( pt->x() != -1 && pt->y() != -1 )
			{
				pt->setX( (int) (pt->x()*m_fRatio/fPreRatio) );
				pt->setY( (int) (pt->y()*m_fRatio/fPreRatio) );
			}
		}
	}
	// 3-2. measure-angle
	if ( m_listPtAngle.count() != 0 )
	{
		QPoint *pt;
		for ( pt=m_listPtAngle.first() ; pt != 0 ; pt=m_listPtAngle.next() )
		{
			pt->setX( (int) (pt->x()*m_fRatio/fPreRatio) );
			pt->setY( (int) (pt->y()*m_fRatio/fPreRatio) );
		}
	}
	// 3-3. ROI-ROI
	if ( m_listPtROI.count() != 0 )
	{
		QPoint *pt;
		for ( pt=m_listPtROI.first() ; pt != 0 ; pt=m_listPtROI.next() )
		{
			if ( pt->x() != -1 && pt->y() != -1 )
			{
				pt->setX( (int) (pt->x()*m_fRatio/fPreRatio) );
				pt->setY( (int) (pt->y()*m_fRatio/fPreRatio) );
			}
		}
	}
} // scaleImage

QList<QRegion> CPlanViewArea::listRgnROI()
{
	return m_listRgnROI;
} // listRgnRegion

void CPlanViewArea::enterDrawMode( /*int mode*/ )
{
//	m_nMeasureMode = mode;
	repaint( false );
} // enterDrawMode

void CPlanViewArea::leaveDrawMode()
{
	repaint( false );
} // leaveDrawMode

void CPlanViewArea::clearMeasureLine()
{
	bool bChanged = false;

	// 1. measure
	if ( m_listPtMeasure.count() != 0 )
	{
		m_listPtMeasure.clear();
		m_listLineValue.clear();	// line value
		m_listAreaValue.clear();	// area value

		bChanged = true;
	}

	// 2. angle
	if ( m_listPtAngle.count() != 0 )
	{
		m_listPtAngle.clear();
		m_listAngleValue.clear();	// angle value

		bChanged = true;
	}

	if ( bChanged )
		repaint( false );
} // clearMeasureLine

void CPlanViewArea::clearROI()
{
	// region of interest
	if ( m_listPtROI.count() != 0 )
	{
		m_listPtROI.clear();
		m_listRgnROI.clear();

		m_pPlanView->slotSliceChanged( m_pPlanView->IDC_SPIN_SLICE->value() );

		// if there 3D model, update 3D model view
		if ( m_pPlanView->mdiWindow()->hasOpenGLView() )
		{
			QMessageBox::information( g_pMainWindow, "Update 3D Model",
				tr( "You have 3D Model on screen.\n\
And Region Of Interest is changed.\n\n\
Now updating 3D Model..!!" ) );

			GLMDIWindow *opengl = m_pPlanView->mdiWindow()->openGLMDIWindow();
			opengl->resetSegmentation();
			opengl->openglView()->updateView();
		}

		repaint( false );
	}
} // clearROI

// private
void CPlanViewArea::calculateMeasureLine()
{
	QPoint *pt[2];
	float *value = new float;

	pt[1] = m_listPtMeasure.last();
	pt[0] = m_listPtMeasure.prev();

	*value = sqrt(
			 ( pt[1]->x()-pt[0]->x() )*( pt[1]->x()-pt[0]->x() ) + 
			 ( pt[1]->y()-pt[0]->y() )*( pt[1]->y()-pt[0]->y() )
			 );

	// have to consider current zoom status
	*value /= m_fRatio;

	m_listLineValue.append( value );
} // calculateMeasureLine

// private
void CPlanViewArea::calculateAngle()
{
	QPoint *pt[3];
	float *value = new float;

	pt[2] = m_listPtAngle.last();
	pt[1] = m_listPtAngle.prev();
	pt[0] = m_listPtAngle.prev();

	double a, b, c;

	a = sqrt(
		( pt[1]->x()-pt[0]->x() )*( pt[1]->x()-pt[0]->x() )	+ 
		( pt[1]->y()-pt[0]->y() )*( pt[1]->y()-pt[0]->y() )
		);
	b = sqrt(
		( pt[2]->x()-pt[1]->x() )*( pt[2]->x()-pt[1]->x() )	+ 
		( pt[2]->y()-pt[1]->y() )*( pt[2]->y()-pt[1]->y() )
		);
	c = sqrt(
		( pt[2]->x()-pt[0]->x() )*( pt[2]->x()-pt[0]->x() )	+ 
		( pt[2]->y()-pt[0]->y() )*( pt[2]->y()-pt[0]->y() )
		);

	*value = acos( ( a*a + b*b - c*c ) / ( 2*a*b ) ) * 180.0 / PI;

	m_listAngleValue.append( value );

	/*
	QString str;
	str.sprintf( "The angle is %f degree.", *value );
	QMessageBox::information( this, tr( "Measured Angle" ), str );
	*/
} // calculateAngle

// private
float CPlanViewArea::lineLength( QPoint pt1, QPoint pt2 )
{
	float value;

	value = sqrt(
			( pt2.x()-pt1.x() )*( pt2.x()-pt1.x() )	+ 
			( pt2.y()-pt1.y() )*( pt2.y()-pt1.y() )
			);

	return value;
} // lineLength

bool CPlanViewArea::isMouseInsideROI( QPoint pt )
{
	int nCount = m_listRgnROI.count();
	bool bContains = true;

	// set bounding rect for display speed
	QList<QRect> boundingRectROI;
	if ( nCount != 0 )
	{
		bContains = false;

		pt.setX( (int) ( pt.x() / m_fRatio ) );
		pt.setY( (int) ( pt.y() / m_fRatio ) );

		int ii;
		for ( ii=0 ; ii<nCount ; ii++ )
		{
			QRect *rt = new QRect( m_listRgnROI.at( ii )->boundingRect() );
			boundingRectROI.append( rt );
		}

		for ( ii=0 ; ii<nCount ; ii++ )
		{
			if ( boundingRectROI.at( ii )->contains( pt ) )
			{
				QRegion *pRgn = m_listRgnROI.at( ii );
						
				if ( pRgn->contains( pt ) )
				{
					bContains = true;
					return bContains;
				}
			}
		}
	}

	return bContains;
} // isMouseInsideROI

// virtual
void CPlanViewArea::paintEvent( QPaintEvent * /*e*/ )
{
	// clear display first
	m_Pixmap.convertFromImage( m_Image );

	// paint all things to display pixmap
	QPen redPen( QColor( 255, 0, 0) , 2 );
	QPen greenPen( QColor( 0, 255, 0 ), 2 );
	QPen bluePen( QColor( 0, 0, 255 ), 2 );

	QPen yellowPen( QColor( 255, 255, 0 ), 2 );
	QPen violetPen( QColor( 255, 0, 255 ), 2 );

	QPen textPen = yellowPen;
	if ( m_pPlanView && !m_pPlanView->volume()->isWhiteIsBone() )
		textPen = bluePen;


	QPainter p;
	QFontMetrics fm( p.font() );
	int height = fm.height();
	p.begin( &m_Pixmap );
		p.setPen( violetPen );

#ifdef _DEMO_VERSION_
	/*************/
	/* draw text */
	/*************/
		QString str;
		str = PROGRAM_NAME;
		str += " ";
		str += PROGRAM_VERSION;
		p.drawText( 5, height+5, str );
		p.drawText( 5, height*2+10, PROGRAM_COPYRIGHT );
		p.drawText( 5, height*3+15, PROGRAM_EMAIL );
#else
	/*********************/
	/* draw slice number */
	/*********************/
		QString str;
		str.sprintf( "%d", m_nSlice );
		p.drawText( 5, height+5, str );
#endif // _DEMO_VERSION_
	p.end();

	/***********/
	/* measure */
	/***********/
	if ( m_listPtMeasure.count() != 0 && m_pPlanView->isMeasureLineShow() )// g_pMainWindow->m_pMeasureLineShowHide->isOn() )
	{
		int nValueCount = 1;
		int nPtCount = 1;

		QPoint *pt;
		float *value;


		QPoint *st_pt = m_listPtMeasure.first();
		QPoint pt1, pt2;
		QPoint mid_pt;

		value = m_listLineValue.first();

		pt1 = *st_pt;

		/*
		QPen redPen( QColor( 255, 0, 0) , 2 );
		QPen yellowPen( QColor( 255, 255, 0 ), 2 );
		*/
		QPainter p;
		p.begin( &m_Pixmap );
		
		for ( pt=m_listPtMeasure.next() ; pt != 0 ; pt=m_listPtMeasure.next() )
		{
			// draw line
			p.setPen( redPen );

			if ( pt->x() != -1 && pt->y() != -1 )
			{
				// draw line
				pt2 = *pt;
				p.drawLine( pt2, pt1 );

				mid_pt.setX( (int) ( ( pt1.x() + pt2.x() ) / 2.0 ) );
				mid_pt.setY( (int) ( ( pt1.y() + pt2.y() ) / 2.0 ) );

				// draw value text
				if ( value )
				{
					QString str;
					str.sprintf( "L%d = %f", nValueCount++, *value );
					p.setPen( textPen );
					p.drawText( mid_pt.x(), mid_pt.y(), str );
					value = m_listLineValue.next();
				}

				pt1 = pt2;
				nPtCount++;
			}
			else
			{
				// if measure line consists of just one line segment
				// don't draw, because of duplication
				if ( nPtCount != 2 )
				{
					mid_pt.setX( (int) ( ( pt1.x() + st_pt->x() ) / 2.0 ) );
					mid_pt.setY( (int) ( ( pt1.y() + st_pt->y() ) / 2.0 ) );

					p.drawLine( pt1, *st_pt );

					// draw value text
					if ( value )
					{
						QString str;
						str.sprintf( "L%d = %f", nValueCount++, *value );
						p.setPen( textPen );
						p.drawText( mid_pt.x(), mid_pt.y(), str );
					}
				}

				value = m_listLineValue.next();

				st_pt = m_listPtMeasure.next();
				if ( st_pt != 0 )
					pt1 = *st_pt;

				nPtCount = 1;
			}
		}

		p.end();	
	}

	/*********/
	/* angle */
	/*********/
	if ( m_listPtAngle.count() != 0 && m_pPlanView->isMeasureLineShow() ) // && g_pMainWindow->m_pMeasureLineShowHide->isOn() )
	{
		int nPtCount = 1;
		int nValueCount = 1;

		QPoint *pt;
		QPoint pt1, pt2;
		QPoint mid_pt;		// for value text
		float *value;

		pt = m_listPtAngle.first();
		pt1 = *pt;

		value = m_listAngleValue.first();

		/*
		QPen greenPen( QColor( 0, 255, 0 ), 2 );
		QPen yellowPen( QColor( 255, 255, 0 ), 2 );
		*/
		QPainter p;
		p.begin( &m_Pixmap );

		for ( pt=m_listPtAngle.next() ; pt != 0 ; pt=m_listPtAngle.next() )
		{
			nPtCount++;

			pt2 = *pt;
			p.setPen( greenPen );
			p.drawLine( pt2, pt1 );
			mid_pt = pt1;	// for value text
			pt1 = pt2;

			if ( (nPtCount % 3) == 0 )
			{
				// draw value
				if ( value )
				{
					QString str;
					str.sprintf( "A%d = %f", nValueCount++, *value );
					p.setPen( textPen );
					p.drawText( mid_pt.x(), mid_pt.y(), str );
					value = m_listAngleValue.next();
				}

				// get next point
				pt = m_listPtAngle.next();
				if ( pt != 0 )
					pt1 = *pt;

				nPtCount++;
			}
		}

		p.end();
	}


	/**********************/
	/* region of interest */
	/**********************/
	if ( m_listPtROI.count() != 0 )
	{
	//	int nValueCount = 1;
		int nPtCount = 1;

		QPoint *pt;
		QPoint *st_pt = m_listPtROI.first();
		QPoint pt1, pt2;

		pt1 = *st_pt;

		/*
		QPen violetPen( QColor( 255, 0, 255 ), 2 );
		*/
		QPainter p;
		p.begin( &m_Pixmap );
		p.setPen( violetPen );
		
		for ( pt=m_listPtROI.next() ; pt != 0 ; pt=m_listPtROI.next() )
		{
			// draw line
			if ( pt->x() != -1 && pt->y() != -1 )
			{
				// draw line
				pt2 = *pt;
				p.drawLine( pt2, pt1 );

				pt1 = pt2;
				nPtCount++;
			}
			else
			{
				p.drawLine( pt1, *st_pt );

				st_pt = m_listPtROI.next();
				if ( st_pt != 0 )
					pt1 = *st_pt;

				nPtCount = 1;
			}
		}

		p.end();	
	}


	/**************/
	/* draw image */
	/**************/
	bitBlt( this, 0, 0, &m_Pixmap );
} // paintEvent

// virtual
void CPlanViewArea::resizeEvent( QResizeEvent * /*e*/ )
{
	resize( m_szImage );
} // resizeEvent

// virtual
void CPlanViewArea::mousePressEvent( QMouseEvent *e )
{
	QPoint *pt;


	/*********************************/
	/* fill erase segmentation popup */
	/*********************************/
	if ( e->button() == Qt::RightButton &&
		( g_pMainWindow->m_pFillSegmentation->isOn() ||
		  g_pMainWindow->m_pEraseSegmentation->isOn() ) )
		m_pFillEraseSizePopup->popup( mapToGlobal( e->pos() ) );

    
	/******************/
	/* measure length */
	/******************/
	if ( e->button() == Qt::LeftButton &&
		 g_pMainWindow->m_pMeasureLength->isOn() )
	{
		// turn measure on
		m_bMeasure = true;

		// append current point
		pt = new QPoint( e->pos() );
		m_listPtMeasure.append( pt );

		if ( m_nPtCount != 0 )
		{
			calculateMeasureLine();
		}
		else
		{
			m_ptMeasureFirst = e->pos();
		}
		
		m_nPtCount++;

		// repaint plan view area
		repaint( false );
	}

	/*****************/
	/* measure angle */
	/*****************/
	if ( e->button() == Qt::LeftButton &&
		 g_pMainWindow->m_pMeasureAngle->isOn() )
	{
		pt = new QPoint( e->pos() );
		m_listPtAngle.append( pt );

		if ( m_nPtCount != 2 )
		{
			m_bMeasure = true;
			m_nPtCount++;
		}
		else
		{
			calculateAngle();

			m_bMeasure = false;
			m_nPtCount = 0;

			g_pMainWindow->m_pMeasureAngle->setOn( false );
			m_pPlanView->leaveDrawMode();
		}

		repaint( false );
	}


	/**********************/
	/* region of interest */
	/**********************/
	if ( e->button() == Qt::LeftButton &&
		 g_pMainWindow->m_pROI->isOn() )
	{
		// turn measure on
		m_bMeasure = true;

		// append current point
		pt = new QPoint( e->pos() );
		m_listPtROI.append( pt );

		if ( m_nPtCount == 0 )
		{
			m_ptMeasureFirst = e->pos();
			m_pPtArray = new QPointArray;
		}

		// ROI
		QPoint p = *pt;
		p.setX( (int) ( p.x() / m_fRatio ) );
		p.setY( (int) ( p.y() / m_fRatio ) );
		m_pPtArray->putPoints( m_nPtCount, 1, p.x(), p.y() );

		m_nPtCount++;

		// repaint plan view area
		repaint( false );
	}


	/*****************************/
	/* fill & erase segmentation */
	/*****************************/
	if ( e->button() == Qt::LeftButton &&
		 isMouseInsideROI( e->pos() ) &&
		(g_pMainWindow->m_pFillSegmentation->isOn() ||
		 g_pMainWindow->m_pEraseSegmentation->isOn() ) )
		m_bFillEraseSegmentation = true;

	m_ptFirst = e->pos();
} // mouePressEvent

// virtual
void CPlanViewArea::mouseReleaseEvent( QMouseEvent * /*e*/ )
{
	/*****************************/
	/* fill & erase segmentation */
	/*****************************/
	m_bFillEraseSegmentation = false;
} // mouseReleaseEvent

// virtual
void CPlanViewArea::mouseMoveEvent( QMouseEvent *e )
{
	/****************************************/
	/* measure length & measure angle & ROI */
	/****************************************/
	if ( m_bMeasure && 
		( g_pMainWindow->m_pMeasureLength->isOn() || 
		  g_pMainWindow->m_pMeasureAngle->isOn() ||
		  g_pMainWindow->m_pROI->isOn() ) )
	{
		// draw virtual line
		QPainter p;
		p.begin( this );
			p.setRasterOp( NotXorROP );
			p.drawLine( m_ptLast, m_ptFirst );
			p.drawLine( e->pos(), m_ptFirst );
		p.end();
	}


	/*****************************/
	/* fill & erase segmentation */
	/*****************************/
	if ( isMouseInsideROI( e->pos() ) && m_bFillEraseSegmentation &&
		( g_pMainWindow->m_pFillSegmentation->isOn() ||
		  g_pMainWindow->m_pEraseSegmentation->isOn() ) )
	{
		CVolume *pVolume = m_pPlanView->volume();

		QPoint pt = e->pos();
		div_t div_result;
		div_result = div( pt.x(), (int) m_fRatio );
		if ( div_result.rem == 0 )
			pt.setX( div_result.quot - 1 );
		else
			pt.setX( div_result.quot );

		div_result = div( pt.y(), (int) m_fRatio );
		if ( div_result.rem == 0 )
			pt.setY( div_result.quot - 1 );
		else
			pt.setY( div_result.quot );

		int i, j;
		for ( i=0 ; i<m_nFillEraseSize ; i++ )
		{
			for ( j=0 ; j<m_nFillEraseSize ; j++ )
			{
				/**********/
				/* volume */
				/**********/

				int x = pt.x() + i;
				int y = pt.y() + j;

				// set voxel value to enable or disable
				int index = x+1 + (m_nHeight-y-1)*m_nWidth + (m_nSlice-1)*(m_nWidth*m_nHeight);

				if ( g_pMainWindow->m_pFillSegmentation->isOn() )
					pVolume->setMask( index, true );
				else
					pVolume->setMask( index, false );


				/*****************/
				/* display image */
				/*****************/

				// get pixel value from original image
				QRgb rgb = m_OriginalImage.pixel( x, y );


				// set display image
				QPoint pt2;
				pt2.setX( (int) (x*m_fRatio) );
				pt2.setY( (int) (y*m_fRatio) );

				// find display color
				int gray, color;
				if ( g_pMainWindow->m_pFillSegmentation->isOn() )
				{
					if ( qRed( rgb ) == 0 && qBlue( rgb ) == 0 && qGreen( rgb ) != 0 )
						color = rgb;
					else
					{
						gray = qGreen( rgb ) + 100;
						if ( gray > 255 ) gray = 255;
						color = qRgb( 0, gray, 0 );
					}
				}
				else
				{
					gray = pVolume->voxelColor( index );
					color = qRgb( gray, gray, gray );
				}

				// change display color
				int pixelWidth = (int) m_fRatio; //*m_nFillEraseSize;
				int curWidth = m_Image.width();
				int curHeight = m_Image.height();
				for ( int ii=0 ; ii<pixelWidth ; ii++ )
				{
					for ( int jj=0 ; jj<pixelWidth ; jj++ )
					{
						if ( pt2.x()+ii <= curWidth && pt2.y()+jj <= curHeight )
							m_Image.setPixel( pt2.x()+ii, pt2.y()+jj, color );
					}
				}
			} // for j
		} // for i

		repaint( false );
	}

	m_ptLast = e->pos();
} // mouseMoveEvent

// virtual
void CPlanViewArea::keyPressEvent( QKeyEvent *e )
{
	if ( m_nPtCount == 0 && e->key() == Qt::Key_Escape )
	{
		// initialize
		m_bMeasure = false;
		g_pMainWindow->m_pMeasureLength->setOn( false );
		g_pMainWindow->m_pROI->setOn( false );

		// repaint plan view area
		repaint( false );

		m_pPlanView->leaveDrawMode();
		return;
	}

	if ( e->key() != Qt::Key_Space )
		return;

	if ( m_bMeasure && g_pMainWindow->m_pMeasureLength->isOn() )		// measure
	{
		// erase unnecessary virtual line
		QPainter p;
		p.begin( this );
			p.setRasterOp( NotXorROP );
			p.drawLine( m_ptLast, m_ptFirst );
		p.end();

		// calculate
		QPoint *pt = m_listPtMeasure.last();
		float *value = new float;
		*value = lineLength( m_ptMeasureFirst, *pt );
		// have to consider zoom status
		*value /= m_fRatio;
		m_listLineValue.append( value );

		// add separator (-1, -1) to point list
		QPoint *sep_pt = new QPoint( -1, -1 );
		m_listPtMeasure.append( sep_pt );

		// initialize
		m_bMeasure = false;
		m_nPtCount = 0;

		// repaint plan view area
		repaint( false );

		g_pMainWindow->m_pMeasureLength->setOn( false );
		m_pPlanView->leaveDrawMode();
	}

	if ( m_bMeasure && g_pMainWindow->m_pROI->isOn() )	// region of interest
	{
		// if points can't be a region, just return
		if ( m_nPtCount <= 2 )
		{
			for ( int i=0 ; i<m_nPtCount ; i++ )
				m_listPtROI.removeLast();

			if ( m_pPtArray )
			{
				delete m_pPtArray;
				m_pPtArray = NULL;
			}

			// initialize
			m_bMeasure = false;
			m_nPtCount = 0;

			// repaint plan view area
			repaint( false );

			g_pMainWindow->m_pROI->setOn( false );
			m_pPlanView->leaveDrawMode();

			return;
		}

		// erase unnecessary virtual line
		QPainter p;
		p.begin( this );
			p.setRasterOp( NotXorROP );
			p.drawLine( m_ptLast, m_ptFirst );
		p.end();

		// add separator (-1, -1) to point list
		QPoint *sep_pt = new QPoint( -1, -1 );
		m_listPtROI.append( sep_pt );

		// ROI
		QRegion *pRgn = new QRegion( *m_pPtArray );
		m_listRgnROI.append( pRgn );
		if ( m_pPtArray )
		{
			delete m_pPtArray;
			m_pPtArray = NULL;
		}

		// initialize
		m_bMeasure = false;
		m_nPtCount = 0;

		// if segmentaion is on, update segmentation
		int nSlice = m_pPlanView->IDC_SPIN_SLICE->value();
		m_pPlanView->slotSliceChanged( nSlice );

		// repaint plan view area
		repaint( false );

		g_pMainWindow->m_pROI->setOn( false );
		m_pPlanView->leaveDrawMode();

		// if there 3D model, update 3D model view
		if ( m_pPlanView->mdiWindow()->hasOpenGLView() )
		{
			QMessageBox::information( g_pMainWindow, "Update 3D Model",
				tr( 
					"You have 3D Model on screen.\n"
					"And Region Of Interest is changed.\n\n"
					"Now updating 3D Model..!!"
				) );

			GLMDIWindow *opengl = m_pPlanView->mdiWindow()->openGLMDIWindow();
			opengl->resetSegmentation();
			opengl->openglView()->updateView();
		}
	}
} // keyPressEvent

// virtual
void CPlanViewArea::focusInEvent( QFocusEvent *e )
{
	QWidget::focusInEvent( e );
} // focusInEvent

// virtual
void CPlanViewArea::focusOutEvent( QFocusEvent *e )
{
	QWidget::focusOutEvent( e );
} // focusOutEvent