#include "VoxelInfoDlg.h"
#include "VolumeFile.h"
#include "MDIWindow.h"
#include "GLMDIWindow.h"
#include "RenderMDIWindow.h"
#include "GLView.h"
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qvalidator.h>

CVoxelInfoDlg::CVoxelInfoDlg( QWidget *parent, const char *name, bool modal, WFlags f )
	: CVoxelInfoDlg_base( parent, name, modal, f )
{
	m_pVolume = NULL;
	
	m_pMDIWindow = NULL;
	m_pGLMDIWindow = NULL;
	m_pRenderMDIWindow = NULL;

	QDoubleValidator *validator = new QDoubleValidator( 0.0, 4096.0, 7, this );
	IDC_EDIT_WIDTH->setValidator( validator );
	IDC_EDIT_HEIGHT->setValidator( validator );
	IDC_EDIT_DEPTH->setValidator( validator );

	connect( IDC_BTN_RESET, SIGNAL( clicked() ), this, SLOT( slotReset() ) );
	connect( IDC_BTN_OK, SIGNAL( clicked() ), this, SLOT( slotOk() ) );
	connect( IDC_BTN_APPLY, SIGNAL( clicked() ), this, SLOT( slotApply() ) );
	connect( IDC_BTN_CANCEL, SIGNAL( clicked() ), this, SLOT( slotCancel() ) );
	connect( IDC_CHECK_NOCHANGE, SIGNAL( stateChanged( int ) ), this, SLOT( slotNoChange() ) );
} // CVoxelInfoDlg

CVoxelInfoDlg::~CVoxelInfoDlg()
{
} // ~CVoxelInfoDlg

enum { NONE_VIEW = 0, PLAN_VIEW, OPENGL_VIEW, RENDER_VIEW };

bool CVoxelInfoDlg::init( QWidget *pWidget )
{
	if ( pWidget && pWidget->isA( "MDIWindow" ) )
	{
		m_pMDIWindow = (MDIWindow *) pWidget;
		m_pVolume = m_pMDIWindow->volume();
		ASSERT( m_pVolume );

		m_nMDIType = PLAN_VIEW;
	}
	else if ( pWidget && pWidget->isA( "GLMDIWindow" ) )
	{
		m_pGLMDIWindow = (GLMDIWindow *) pWidget;
		m_pVolume = m_pGLMDIWindow->volume();
		ASSERT( m_pVolume );

		m_pGLMDIWindow->setVoxelInfoDlg( this );

		m_nMDIType = OPENGL_VIEW;
	}
	else if ( pWidget && pWidget->isA( "RenderMDIWindow" ) )
	{
		m_pRenderMDIWindow = (RenderMDIWindow *) pWidget;
		m_pVolume = m_pRenderMDIWindow->volume();
		ASSERT( m_pVolume );

		m_nMDIType = RENDER_VIEW;
	}

	QString str;

	// set original voxel size
	float fWidth = m_pVolume->voxelOriginWidth();
	float fHeight = m_pVolume->voxelOriginHeight();
	float fDepth = m_pVolume->voxelOriginDepth();

	str.sprintf( "%f", fWidth );
	IDC_LBL_WIDTH->setText( str );
	str.sprintf( "%f", fHeight );
	IDC_LBL_HEIGHT->setText( str );
	str.sprintf( "%f", fDepth );
	IDC_LBL_DEPTH->setText( str );

	// set voxel size
	m_fOldWidth = m_pVolume->voxelWidth();
	m_fOldHeight = m_pVolume->voxelHeight();
	m_fOldDepth = m_pVolume->voxelDepth();

	m_fNewWidth = m_fOldWidth;
	m_fNewHeight = m_fOldHeight;
	m_fNewDepth = m_fOldDepth;

	str.sprintf( "%f", m_fOldWidth );
	IDC_EDIT_WIDTH->setText( str );
	str.sprintf( "%f", m_fOldHeight );
	IDC_EDIT_HEIGHT->setText( str );
	str.sprintf( "%f", m_fOldDepth );
	IDC_EDIT_DEPTH->setText( str );

	// set check box initially unchecked. it means user can't change voxel size
	IDC_CHECK_NOCHANGE->setChecked( true );

	if ( m_nMDIType != OPENGL_VIEW )
	{
		IDC_CHECK_NOCHANGE->setEnabled( false );
		IDC_BTN_RESET->setEnabled( false );
		IDC_BTN_APPLY->setEnabled( false );
		IDC_BTN_CANCEL->setEnabled( false );
	}

	return true;
} // init

// slot
void CVoxelInfoDlg::slotNoChange()
{
	if ( IDC_CHECK_NOCHANGE->isOn() )
	{
		IDC_EDIT_WIDTH->setEnabled( false );
		IDC_EDIT_HEIGHT->setEnabled( false );
		IDC_EDIT_DEPTH->setEnabled( false );

		IDC_BTN_RESET->setEnabled( false );
		IDC_BTN_APPLY->setEnabled( false );
		IDC_BTN_CANCEL->setEnabled( false );
	}
	else
	{
		IDC_EDIT_WIDTH->setEnabled( true );
		IDC_EDIT_HEIGHT->setEnabled( true );
		IDC_EDIT_DEPTH->setEnabled( true );

		IDC_BTN_RESET->setEnabled( true );
		IDC_BTN_APPLY->setEnabled( true );
		IDC_BTN_CANCEL->setEnabled( true );
	}
} // slotChange

// slot
void CVoxelInfoDlg::slotReset()
{
	// set voxel size
	QString str;
	str.sprintf( "%f", m_fOldWidth );
	IDC_EDIT_WIDTH->setText( str );
	str.sprintf( "%f", m_fOldHeight );
	IDC_EDIT_HEIGHT->setText( str );
	str.sprintf( "%f", m_fOldDepth );
	IDC_EDIT_DEPTH->setText( str );
} // slotReset

// slot
void CVoxelInfoDlg::slotOk()
{
	slotApply();

	if ( m_pGLMDIWindow )
		m_pGLMDIWindow->unsetVoxelInfoDlg();

	QDialog::accept();
} // slotOk

// slot
void CVoxelInfoDlg::slotApply()
{
	QString str;
	float fWidth, fHeight, fDepth;

	str = IDC_EDIT_WIDTH->text();
	fWidth = str.toDouble();
	str = IDC_EDIT_HEIGHT->text();
	fHeight = str.toDouble();
	str = IDC_EDIT_DEPTH->text();
	fDepth = str.toDouble();

	if ( m_nMDIType == OPENGL_VIEW && 
		( fWidth != m_fNewWidth || fHeight != m_fNewHeight || fDepth != m_fNewDepth ) )
	{
		m_pVolume->setVoxelWidth( fWidth );
		m_pVolume->setVoxelHeight( fHeight );
		m_pVolume->setVoxelDepth( fDepth );

		m_pGLMDIWindow->openglView()->updateView();

		m_fNewWidth = fWidth;
		m_fNewHeight = fHeight;
		m_fNewDepth = fDepth;
	}
} // slotApply

// slot
void CVoxelInfoDlg::slotCancel()
{
	/*
	if ( m_pMDIWindow )
	{
	}
	*/
	if ( m_pGLMDIWindow )
	{
		m_pGLMDIWindow->unsetVoxelInfoDlg();
	}
	/*
	if ( m_pRenderMDIWindow )
	{
	}
	*/

	QDialog::reject();
} // slotCancel

// protected
void CVoxelInfoDlg::closeEvent( QCloseEvent * ) 
{
	slotCancel();
} // closeEvent