/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include "MainWindow.h"
#include "MDIWindow.h"
#include "VolumeFile.h"
#include "PlanView.h"
#include "GLMDIWindow.h"
#include "RenderMDIWindow.h"
#include "MontageMDIWindow.h"
#include "MarchingCubeMDIWindow.h"
#include "SegmentationDlg.h"
#include "VoxelInfoDlg.h"
#include "ResamplingDlg.h"
#include "MaterialPropertyDlg.h"
#include <qdir.h>
#include <qfiledialog.h>
#include <qtoolbutton.h>
#include <qcursor.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qspinbox.h>

#include <qmessagebox.h>	//XXX



////////////////////////////////////////////////////////////////////////////
//
// MDIWindow
//

MDIWindow::MDIWindow( QWidget *parent, const char *name, WFlags f )
	: QMainWindow( parent, name, f )
{
	m_pVolume = NULL;
	m_pGLMDIWindow = NULL;
	m_pRenderMDIWindow = NULL;
	m_pMontageMDIWindow = NULL;
	m_pMarchingCubeMDIWindow = NULL;
	m_pSegmentationDlg = NULL;
		
	// plan view
	m_pPlanView = new CPlanView( this );
	setFocusProxy( m_pPlanView );
	setCentralWidget( m_pPlanView );

	/*
	setMouseTracking( true );	// enable mouseMoveEvent
	setFocusPolicy( QWidget::StrongFocus );	// enable keyPressEvent
	*/
} /* MDIWindow */

MDIWindow::~MDIWindow()
{
//	g_pMainWindow->closeMDIWindow( m_strVolumeFileName );

	if ( m_pPlanView )
	{
		delete m_pPlanView;
		m_pPlanView = NULL;
	}

	if ( m_pGLMDIWindow )
	{
		delete m_pGLMDIWindow;
		m_pGLMDIWindow = NULL;
	}

	if ( m_pRenderMDIWindow )
	{
		delete m_pRenderMDIWindow;
		m_pRenderMDIWindow = NULL;
	}

	if ( m_pMontageMDIWindow )
	{
		delete m_pMontageMDIWindow;
		m_pMontageMDIWindow = NULL;
	}

	if ( m_pMarchingCubeMDIWindow )
	{
		delete m_pMarchingCubeMDIWindow;
		m_pMarchingCubeMDIWindow = NULL;
	}

	if ( m_pSegmentationDlg )
	{
		delete m_pSegmentationDlg;
		m_pSegmentationDlg = NULL;
	}

	if ( m_pVolume )
	{
		delete m_pVolume;
		m_pVolume = NULL;
	}
} /* ~MDIWindow */

CVolume *MDIWindow::volume()
{
	if ( m_pVolume )
		return m_pVolume;
	else
		return NULL;
} /* volume */

int MDIWindow::currentSlice()
{
	if ( m_pPlanView )
		return m_pPlanView->currentSlice();
	else
		return -1;
} /* currentSlice */

CPlanView *MDIWindow::planView()
{
	if ( m_pPlanView )
		return m_pPlanView;
	else
		return NULL;
} /* planView */

bool MDIWindow::open( QString fileName )
{
	m_pVolume = new CVolume( this );
	if ( !m_pVolume->open( fileName ) )
		return false;

	m_strVolumeFileName = fileName;

	// plan view
	m_pPlanView->setVolume( m_pVolume );
	setCaption( fileName );

	resize( 512, 512 );

	return true;
} /* open */

bool MDIWindow::save( QString fileName )
{
	if ( !m_pVolume )
		return false;

	// save volume on disk
	if ( !m_pVolume->save( fileName ) )
		return false;

	// set caption of all view
	setCaption( fileName );
	if ( hasOpenGLView() )
		m_pGLMDIWindow->setCaption( tr( "3D Model : " ) + fileName );
	if ( hasRenderView() )
		m_pRenderMDIWindow->setCaption( tr( "Render View : " ) + fileName );
	if ( hasMontageView() )
		m_pMontageMDIWindow->setCaption( tr( "Montage View : " ) + fileName );
	if ( hasMarchingCubeView() )
		m_pMarchingCubeMDIWindow->setCaption( tr( "Surface Model : " ) + fileName );

	return true;
} /* save */

bool MDIWindow::importStack( KEYWORD key )
{
	m_pVolume = new CVolume( this );
	if ( !m_pVolume->init( key ) )
	{
		/*
		QMessageBox::critical( g_pMainWindow, "Volume File Error",
			"Can't generate Volume..." );
		*/

		return false;
	}

	// plan view
	m_pPlanView->setVolume( m_pVolume );

	resize( 512, 512 );

	return true;
} /* importStack */

bool MDIWindow::exportToAnsys()
{
	if ( !m_pVolume )
		return false;

	// show material property dialog
	CMaterialPropertyDlg dlg( this, NULL, true );
	if ( !dlg.init( m_pVolume ) )
		return false;

	if ( dlg.exec() != QDialog::Accepted )
		return false;

	// if user selected ok button on material property dialog

	if ( !m_pVolume->exportToAnsys( dlg.IDC_EDIT_FILENAME->text() ) )
		return false;

	QMessageBox::information( this, "Export Info",
		tr( "Export to Ansys Done..!!" ) );

	return true;
} /* exportToAnsys */

bool MDIWindow::exportToSurface( QString fileName, int type )
{
	if ( !m_pVolume )
		return false;

	if ( !m_pVolume->exportToSurface( fileName, type ) )
		return false;

	QMessageBox::information( this, "Export Info",
		tr( "Export to Surface Done..!!" ) );

	return true;
} /* exportToSurface */

bool MDIWindow::hasOpenGLView()
{
	if ( m_pGLMDIWindow )
		return true;
	else
		return false;
} /* hasOpenGLView */

bool MDIWindow::hasRenderView()
{
	if ( m_pRenderMDIWindow )
		return true;
	else
		return false;
} /* hasRenderView */

bool MDIWindow::hasMontageView()
{
	if ( m_pMontageMDIWindow )
		return true;
	else
		return false;
} /* hasMontageView */

bool MDIWindow::hasMarchingCubeView()
{
	if ( m_pMarchingCubeMDIWindow )
		return true;
	else
		return false;
} /* hasMarchingCubeView */

bool MDIWindow::hasSegmentationDlg()
{
	if ( m_pSegmentationDlg )
		return true;
	else
		return false;
} /* hasSegmentationDlg */

/*
bool MDIWindow::hasVoxelInfoDlg()
{
	if ( m_pVoxelInfoDlg )
		return true;
	else
		return false;
} /* hasVoxelInfoDlg */

void MDIWindow::setOpenGLView( GLMDIWindow *pGLMDIWindow )
{
	m_pGLMDIWindow = pGLMDIWindow;
	ASSERT( m_pGLMDIWindow );
} /* setOpenGLView */

void MDIWindow::unsetOpenGLView()
{
	m_pGLMDIWindow = NULL;
} /* unsetOpenGLView */

void MDIWindow::setRenderView( RenderMDIWindow *pRenderMDIWindow )
{
	m_pRenderMDIWindow = pRenderMDIWindow;
	ASSERT( m_pRenderMDIWindow );
} /* setRenderView */

void MDIWindow::unsetRenderView()
{
	m_pRenderMDIWindow = NULL;
} /* unsetRenderView */

void MDIWindow::setMontageView( MontageMDIWindow *pMontageMDIWindow )
{
	m_pMontageMDIWindow = pMontageMDIWindow;
	ASSERT( m_pMontageMDIWindow );
} /* setMontageView */

void MDIWindow::unsetMontageView()
{
	m_pMontageMDIWindow = NULL;
} /* unsetMontageView */

void MDIWindow::setMarchingCubeView( MarchingCubeMDIWindow *pMarchingCubeMDIWindow )
{
	m_pMarchingCubeMDIWindow = pMarchingCubeMDIWindow;
	ASSERT( m_pMarchingCubeMDIWindow );
} /* setMarchingCubeView */

void MDIWindow::unsetMarchingCubeView()
{
	m_pMarchingCubeMDIWindow = NULL;
} /* unsetMarchingCubeView */

void MDIWindow::setSegmentationDlg( CSegmentationDlg *pSegmentationDlg )
{
	m_pSegmentationDlg = pSegmentationDlg;
	ASSERT( m_pSegmentationDlg );
} /* setSegmentationDlg */

void MDIWindow::unsetSegmentationDlg()
{
	m_pSegmentationDlg = NULL;
} /* unsetSegmentationDlg */

GLMDIWindow *MDIWindow::openGLMDIWindow()
{
	if ( m_pGLMDIWindow )
		return m_pGLMDIWindow;
	else
		return NULL;
} /* openGLMDIWindow */

RenderMDIWindow *MDIWindow::renderMDIWindow()
{
	if ( m_pRenderMDIWindow )
		return m_pRenderMDIWindow;
	else
		return NULL;
} /* renderMDIWindow */

MontageMDIWindow *MDIWindow::montageMDIWindow()
{
	if ( m_pMontageMDIWindow )
		return m_pMontageMDIWindow;
	else
		return NULL;
} /* montageMDIWindow */

MarchingCubeMDIWindow *MDIWindow::marchingCubeMDIWindow()
{
	if ( m_pMarchingCubeMDIWindow )
		return m_pMarchingCubeMDIWindow;
	else
		return NULL;
} /* marchingCubeMDIWindow */

bool MDIWindow::resampling()
{
	CResamplingDlg dlg( this, NULL, true );
	if ( dlg.init( this ) )
	{
		if ( dlg.exec() != QDialog::Accepted )
			return false;

		QString strDir = dlg.IDC_EDIT_DIRECTORY->text();
		QDir dir( strDir );
		if ( !dir.exists() )
			dir.mkdir( strDir );

		QString strKeyword = dlg.IDC_EDIT_KEYWORD->text();
		QString strType = dlg.IDC_COMBO_FILEFORMAT->currentText();
		int nViewType = dlg.IDC_COMBO_VIEWTYPE->currentItem();

		WORD wStart = dlg.IDC_SPIN_START->value();
		WORD wEnd = dlg.IDC_SPIN_END->value();

		if ( !m_pPlanView->resampling( strType, strDir, strKeyword,	nViewType, wStart, wEnd ) )
			return false;

		QMessageBox::information( this, "Resampling Info",
			tr( "Resampling Done..!!" ) );
	}
	else
		return false;

	return true;
} /* resampling */

// virtual
void MDIWindow::closeEvent( QCloseEvent *e )
{
	if ( m_pPlanView->isTimerOn() )
	{
		e->ignore();
		return;
	}

	if ( hasSegmentationDlg() )
	{
		e->ignore();
		return;
	}

	bool bOk = true;
	if ( hasMontageView() || hasOpenGLView() || hasRenderView() || hasMarchingCubeView() )
	{
		switch ( QMessageBox::information( this, "Close Plan View",
				tr( "Plan View has(have) other view(s).\n\nDo you want to close ?" ),
				"OK", "Cancel", 0, 0, 1 ) )
		{
		case 0:		// OK
			bOk = true;
			break;
		case 1:		// Cancel
			bOk = false;
			break;
		}
	}

	if ( !bOk )
	{
		e->ignore();
		return;
	}

	ASSERT( g_pMainWindow );

	g_pMainWindow->m_pROI->setOn( false );

	g_pMainWindow->m_pMeasureLength->setOn( false );
	g_pMainWindow->m_pMeasureAngle->setOn( false );
//	g_pMainWindow->m_pMeasureRegion->setOn( false );
	g_pMainWindow->m_pMeasureLineShowHide->setOn( false );

	// rest segmentatin show/hide toolbutton
	g_pMainWindow->m_pSegmentationShowHide->setOn( false );

	g_pMainWindow->m_pFillSegmentation->setOn( false );
	g_pMainWindow->m_pEraseSegmentation->setOn( false );

	g_pMainWindow->closeMDIWindow( m_strVolumeFileName );

	e->accept();
} /* closeEvent */

bool MDIWindow::event( QEvent *e )
{
	return QMainWindow::event( e );
}  /* event */