/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include "MaterialPropertyDlg.h"
#include "VolumeFile.h"
#include "MathExpParser.h"	// math expression parser
#include <qlineedit.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qregexp.h>
#include <qmessagebox.h>
#include <qfiledialog.h>

CMaterialPropertyDlg::CMaterialPropertyDlg( QWidget *parent, const char *name, bool modal, WFlags f )
	: CMaterialPropertyDlg_base( parent, name, modal, f )
{
	connect( IDC_BTN_FILENAME, SIGNAL( clicked() ),
		this, SLOT( slotFileName() ) );
	connect( IDC_EDIT_FILENAME, SIGNAL( textChanged ( const QString & ) ),
		this, SLOT( slotFileNameChanged( const QString & ) ) );

	connect( IDC_EDIT_POSSION, SIGNAL( textChanged( const QString & ) ),
		this, SLOT( slotEditChanged() ) );
	connect( IDC_EDIT_DENSITY, SIGNAL( textChanged( const QString & ) ),
		this, SLOT( slotEditChanged() ) );
	connect( IDC_EDIT_YOUNG, SIGNAL( textChanged( const QString & ) ),
		this, SLOT( slotEditChanged() ) );

	connect( IDC_BTN_OK, SIGNAL( clicked() ), this, SLOT( slotOk() ) );
	connect( IDC_BTN_CANCEL, SIGNAL( clicked() ), this, SLOT( slotCancel() ) );

	IDC_GROUP_MATERIALPROPERTY->setEnabled( false );
	IDC_BTN_OK->setEnabled( false );
} /* CMaterialPropertyDlg */

CMaterialPropertyDlg::~CMaterialPropertyDlg()
{
} /* ~CMaterialPropertyDlg */

bool CMaterialPropertyDlg::init( CVolume *pVolume )
{
	m_pVolume = pVolume;
	ASSERT( m_pVolume );

	IDC_LBL_DESCRIPTION->setText(
		tr( 
"You can use [mathematical expression] for material property.\n\n\
ex)\n\
       density = value / 255 * 2.5 + 1.5\n\
       young's modulus = 2845 * (10^6) * (density^3) + sin(value)\n\n\
-> value is voxel value & density can be used young's modulus.\n\
=> Refer to manual to check available function such as sin(), cos()..."
		) );

	return true;
} /* init */

// slot
void CMaterialPropertyDlg::slotFileName()
{
	// file extension
	QString fn = QFileDialog::getSaveFileName( NULL, "*.ans", this );
    if ( !fn.isEmpty() )
	{
		// check file extension
		if ( fn.find( ".ans", 0 ) != -1 || fn.find( ".ANS", 0 ) != -1 )
			;
		else
			fn.append( ".ans" );

		IDC_EDIT_FILENAME->setText( fn );
	}
} /* slotFileName */

// slot
void CMaterialPropertyDlg::slotFileNameChanged( const QString &str )
{
	if ( str.isEmpty() )
		IDC_GROUP_MATERIALPROPERTY->setEnabled( false );
	else
		IDC_GROUP_MATERIALPROPERTY->setEnabled( true );

	slotEditChanged();
} /* slotFileNameChanged */

// slot
void CMaterialPropertyDlg::slotEditChanged()
{
	QString str1, str2, str3, str4;
	str1 = IDC_EDIT_POSSION->text();
	str2 = IDC_EDIT_DENSITY->text();
	str3 = IDC_EDIT_YOUNG->text();
	str4 = IDC_EDIT_FILENAME->text();

	if ( str1.isEmpty() || str2.isEmpty() || str3.isEmpty() )
		IDC_BTN_OK->setEnabled( false );
	else
	{
		if ( str4.isEmpty() )
			IDC_BTN_OK->setEnabled( false );
		else
			IDC_BTN_OK->setEnabled( true );
	}
} /* slotEditChanged */

// slot
void CMaterialPropertyDlg::slotOk()
{
	CMathExpParser mep;

	QString errMsg;
	double possion, density, young;
	double fDensity[256];
	int error;

	QString str1 = IDC_EDIT_POSSION->text();
	QString str2 = IDC_EDIT_DENSITY->text();
	QString str3 = IDC_EDIT_YOUNG->text();
	int n1 = str1.find( "value" );
	int n2 = str2.find( "value" );
	int n3 = str3.find( "value" );
	int d3 = str3.find( "density" );

	QString str, tmpStr;

	// possion ratio
	if ( n1 == -1 )
	{
		if ( !mep.parse( str1, &possion, &error ) )
		{
			errMsg.sprintf( "Error on Possion Ratio : %d", error );
			QMessageBox::critical( this, "Error", errMsg );

			return;
		}

		m_pVolume->setPossionLUT( 0, possion );
		m_pVolume->setHomogeneousPossion( true );
	}
	else
	{
		for ( int i=0 ;  i<256 ; i++ )
		{
			str.sprintf( "%d", i );
			tmpStr = str1;
			str = tmpStr.replace( QRegExp( "value" ), str );
			if ( !mep.parse( str, &possion, &error ) )
			{
				errMsg.sprintf( "Error on Possion Ratio : %d", error );
				QMessageBox::critical( this, "Error", errMsg );

				return;
			}

			m_pVolume->setPossionLUT( i, possion );
		}

		m_pVolume->setHomogeneousPossion( false );
	}
	// density
	if ( n2 == -1 )
	{
		if ( !mep.parse( str2, &density, &error ) )
		{
			errMsg.sprintf( "Error on Density : %d", error );
			QMessageBox::critical( this, "Error", errMsg );

			return;
		}

		m_pVolume->setDensityLUT( 0, density );
		m_pVolume->setHomogeneousDensity( true );
	}
	else
	{
		for ( int i=0 ;  i<256 ; i++ )
		{
			str.sprintf( "%d", i );
			tmpStr = str2;
			str = tmpStr.replace( QRegExp( "value" ), str );
			if ( !mep.parse( str, &density, &error ) )
			{
				errMsg.sprintf( "Error on Density : %d", error );
				QMessageBox::critical( this, "Error", errMsg );

				return;
			}

			m_pVolume->setDensityLUT( i, density );
			fDensity[i] = density;
		}

		m_pVolume->setHomogeneousDensity( false );
	}
	// young's modulus
	if ( n3 == -1 && d3 == -1 )		// there is no "value" or "density"
	{
		if ( !mep.parse( str3, &young, &error ) )
		{
			errMsg.sprintf( "Error on Young's Modulus : %d", error );
			QMessageBox::critical( this, "Error", errMsg );

			return;
		}

		m_pVolume->setYoungLUT( 0, young );
		m_pVolume->setHomogeneousYoung( true );
	}
	else
	{
		for ( int i=0 ;  i<256 ; i++ )
		{
			str.sprintf( "%d", i );
			tmpStr = str3;
			tmpStr = tmpStr.replace( QRegExp( "value" ), str );
			str.sprintf( "%f", fDensity[i] );
			str = tmpStr.replace( QRegExp( "density" ), str );
			if ( !mep.parse( str, &young, &error ) )
			{
				errMsg.sprintf( "Error on Young's Modulus : %d", error );
				QMessageBox::critical( this, "Error", errMsg );

				return;
			}

			m_pVolume->setYoungLUT( i, young );
		}

		m_pVolume->setHomogeneousYoung( false );
	}

	QDialog::accept();
} /* slotOk */

// slot
void CMaterialPropertyDlg::slotCancel()
{
	QDialog::reject();
} /* slotCancel */
