/************************************************************************/
/* MGIBA: Mesh Generator for Image Based Analysis                       */
/* ==============================================                       */
/*                                                                      */
/* Written by Yongchul Kim (blksaint@freechal.com)                      */
/* http://www.diniworld.com                                             */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License.       */
/*                                                                      */
/* Please read LICENSE.txt                                              */
/************************************************************************/


#include "MontageView.h"
#include "MontageMDIWindow.h"	
#include "MDIWindow.h"
#include "PlanView.h"
#include "MainWindow.h"
#include "VolumeFile.h"

#include <stdlib.h>		// div_t
#include <qapplication.h>
#include <qpen.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qstatusbar.h>
#include <qimage.h>
#include <qprogressdialog.h>
#include <qscrollview.h>


////////////////////////////////////////////////////////////////////////////
//
// CMontageView
//
	
CMontageView::CMontageView( QWidget *parent, const char *name, WFlags f )
	: QWidget( parent, name, f )
{
	m_pMontageMDIWindow = (MontageMDIWindow *) parent;
	ASSERT( m_pMontageMDIWindow );

	m_pVolume = NULL;
	m_pMontageViewArea = NULL;
	m_pScrollView = NULL;
	m_pMontageViewArea = NULL;

	m_nType = 3;	// z-view

	connect( this, SIGNAL( viewResized() ), 
		this, SLOT( slotMoveImageToCenter() ) );
} /* CMontageView */

CMontageView::~CMontageView()
{
	if ( m_pMontageViewArea )
	{
		delete m_pMontageViewArea;
		m_pMontageViewArea = NULL;
	}

	if ( m_pScrollView )
	{
		delete m_pScrollView;
		m_pScrollView = NULL;
	}
} /* ~CMontageView */
	
bool CMontageView::init( CVolume *pVolume, int nSlice )
{
	m_pVolume = pVolume;
	ASSERT( m_pVolume );

	// 1. scroll view
	m_pScrollView = new QScrollView( this );
	ASSERT( m_pScrollView );

	// 2. scroll area
	m_pMontageViewArea = new CMontageViewArea( this );
	QPixmap pixmap;
	makeImage( pixmap );
	m_pMontageViewArea->init( pixmap, m_nUnitHeight, nSlice );	// z-view : XXX
	m_pScrollView->addChild( m_pMontageViewArea );

	return true;
} /* init */

MontageMDIWindow *CMontageView::montageMDIWindow()
{
	if ( m_pMontageMDIWindow )
		return m_pMontageMDIWindow;
	else
		return NULL;
} /* montageMDIWindow */
			
CVolume *CMontageView::volume()
{
	if ( m_pVolume )
		return m_pVolume;
	else
		return NULL;
} /* volume */

CMontageViewArea *CMontageView::montageViewArea()
{
	if ( m_pMontageViewArea )
		return m_pMontageViewArea;
	else
		return NULL;
} /* montageViewArea */

// slot
void CMontageView::slotMoveImageToCenter()
{
	QScrollBar *vb = m_pScrollView->verticalScrollBar();
	QScrollBar *hb = m_pScrollView->horizontalScrollBar();

	int x = 0;
	int y = 0;
	int w = m_pScrollView->width();
	int h = m_pScrollView->height();

	if ( !hb->isVisible() && m_pMontageViewArea->width() < w )
		x = (int) ( ( w - m_pMontageViewArea->width() ) / 2 ) - 1;

	if ( !vb->isVisible() && m_pMontageViewArea->height() < h )
		y = (int) ( ( h - m_pMontageViewArea->height() ) / 2 ) - 1;

	m_pScrollView->moveChild( m_pMontageViewArea, x, y );
} /* slotMoveImageToCenter */

// private
bool CMontageView::makeImage( QPixmap &pixmap )
{
	WORD i, j, k;

	g_pMainWindow->statusBar()->message( tr( "Generate Montage View..." ) );

	// set geometry of virtual volume according to view type
	if ( m_nType == 1 )			// x-view
	{
		m_nWidth  = m_pVolume->volumeHeight();
		m_nHeight = m_pVolume->volumeDepth();
		m_nDepth  = m_pVolume->volumeWidth();
	}
	else if ( m_nType == 2 )	// y-view
	{
		m_nWidth  = m_pVolume->volumeWidth();
		m_nHeight = m_pVolume->volumeDepth();
		m_nDepth  = m_pVolume->volumeHeight();
	}
	else if ( m_nType == 3 )	// z-view
	{
		m_nWidth  = m_pVolume->volumeWidth();
		m_nHeight = m_pVolume->volumeHeight();
		m_nDepth  = m_pVolume->volumeDepth();
	}

	// calculate montage view size
	int numRow;
//	int unitHeight;
	int index, gray;
	div_t div_result;

	div_result = div( m_nDepth, NUM_UNIT_MONTAGE );

	numRow = div_result.quot;
	if ( div_result.rem != 0 )
		numRow++;

	m_nUnitHeight = (int) ( ( (float)m_nHeight / (float)m_nWidth ) * UNIT_MONTAGE_SIZE );

	QImage image( NUM_UNIT_MONTAGE*UNIT_MONTAGE_SIZE, numRow*m_nUnitHeight, 32 );

    QProgressDialog progress( "Make Montage View...", "Abort", m_nDepth,
                                this, "progress", true );
	progress.show();

	for ( i=1 ; i<=m_nDepth ; i++ )
	{
		progress.setProgress( i );
		qApp->processEvents();

		if ( progress.wasCancelled() )
		{
			g_pMainWindow->statusBar()->message( tr( "Ready" ) );
			return false;
		}

		QImage unitImage( m_nWidth, m_nHeight, 32 );

		for ( j=1 ; j<=m_nHeight; j++ )
		{
			for ( k=1 ; k<=m_nWidth ; k++ )
			{
				index = voxelIndex( k, j, i );
				gray = (int) m_pVolume->voxelColor( index );
				unitImage.setPixel( k-1, j-1, qRgb( gray, gray, gray ) );
			}
		}

		// find the position on montage view
		div_result = div( i, NUM_UNIT_MONTAGE );

		if ( div_result.rem == 0 )
		{
			div_result.quot--;
			div_result.rem = NUM_UNIT_MONTAGE;
		}

		unitImage = unitImage.smoothScale( UNIT_MONTAGE_SIZE, m_nUnitHeight );
		for ( j=0 ; j<UNIT_MONTAGE_SIZE ; j++ )
		{
			for ( k=0 ; k<m_nUnitHeight ; k++ )
			{
				image.setPixel(
					( div_result.rem-1 )*UNIT_MONTAGE_SIZE + j,
					( div_result.quot )*m_nUnitHeight + k, 
					unitImage.pixel( j, k ) 
					);
			}
		}
	}
	progress.setProgress( m_nWidth );
	progress.hide();

	pixmap.convertFromImage( image );

	g_pMainWindow->statusBar()->message( tr( "Ready" ) );

	return true;
} /* makeImage */

// private
DWORD CMontageView::voxelIndex( WORD x, WORD y, int slice )
{
	int index;
	if ( m_nType == 1 )			// x-view
		index = y + (x-1)*m_nDepth + (m_nHeight-y)*(m_nWidth*m_nDepth-1) + (slice-1);
	else if ( m_nType == 2 )	// y-view
		index = x + (m_nHeight-y)*(m_nWidth*m_nDepth) + (slice-1)*m_nWidth;
	else if ( m_nType == 3 )	// z-view
		index = x + (m_nHeight-y)*m_nWidth + (slice-1)*(m_nWidth*m_nHeight);

	return index;
} /* voxelIndex */

// protected
void CMontageView::resizeEvent( QResizeEvent * /*e*/ )
{
	m_pScrollView->setGeometry( 0, 0, width(), height() );

	emit viewResized();
} /* resizeEvent */

// slots
void CMontageView::slotSetSlice( int slice )
{
	m_pMontageViewArea->slotSetSlice( slice );
} /* slotSetSlice */

// slots
void CMontageView::slotTypeChanged( int type )
{
	m_nType = type;

	QPixmap pixmap;
	makeImage( pixmap );
	m_pMontageViewArea->init( pixmap, m_nUnitHeight, 1 );

	emit viewResized();
} /* slotTypeChanged */


////////////////////////////////////////////////////////////////////////////
//
// CMontageViewArea
//

CMontageViewArea::CMontageViewArea( QWidget *parent, const char *name, WFlags f )
	: QWidget( parent, name, f )
{
	m_pMontageView = (CMontageView *) parent;
	ASSERT( m_pMontageView );

	setBackgroundMode( NoBackground );

	setMouseTracking( true );				// enable mouseMoveEvent
	setFocusPolicy( QWidget::StrongFocus );	// enable keyPressEvent	
} /* CMontageViewArea */

CMontageViewArea::~CMontageViewArea()
{
} /* ~CMontageViewArea */

bool CMontageViewArea::init( QPixmap pixmap, int unitHeight, int nSlice )
{
	m_OriPixmap = pixmap;
	m_Pixmap = m_OriPixmap;

	m_nUnitHeight = unitHeight;
	m_nSlice = nSlice;

	calculateRowCol( nSlice );

	resize( m_Pixmap.width(), m_Pixmap.height() );

	int nCapX = (int) ( (float)m_Pixmap.width() / (float)NUM_UNIT_MONTAGE );
	int nCapY = m_nUnitHeight;
	int nNumX = m_Pixmap.width() / nCapX;
	int nNumY = m_Pixmap.height() / nCapY;

	m_nMaxSlice = nNumX * nNumY;

	return true;
} /* init */

// protected
void CMontageViewArea::paintEvent( QPaintEvent * )
{
	QString str;

#ifdef _DEMO_VERSION_
	/*************/
	/* draw text */
	/*************/
	QPen whitePen( QColor( 255, 255, 255 ), 2 );

	QPainter p;
	QFontMetrics fm( p.font() );
	int height = fm.height();
	p.begin( &m_Pixmap );
		p.setPen( whitePen );
		str = PROGRAM_NAME;
		str += " ";
		str += PROGRAM_VERSION;
		p.drawText( 5, height+5, str );
		p.drawText( 5, height*2+10, PROGRAM_COPYRIGHT );
		p.drawText( 5, height*3+15, PROGRAM_EMAIL );
	p.end();
#endif // _DEMO_VERSION_

	int nCapX = (int) ( (float)m_Pixmap.width() / (float)NUM_UNIT_MONTAGE );
	int nCapY = m_nUnitHeight;
	int nNumX = m_Pixmap.width() / nCapX;
	int nNumY = m_Pixmap.height() / nCapY;

	QPen linePen;
	QPen textPen;
	QPen cursorPen;
	QPen cursorTextPen;

	linePen.setWidth( 1 );
	textPen.setWidth( 1 );
	cursorPen.setWidth( 3 );
	cursorTextPen.setWidth( 1 );
	
	linePen.setColor( QColor( 255, 255, 0 ) );
	textPen.setColor( QColor( 0, 255, 0 ) );
	cursorPen.setColor( QColor( 0, 255, 255 ) );
	cursorTextPen.setColor( QColor( 255, 0, 255 ) );

	if ( !m_pMontageView->volume()->isWhiteIsBone() )
	{
		linePen.setColor( QColor( 0, 0, 255 ) );
		textPen.setColor( QColor( 255, 0, 0 ) );
		cursorPen.setColor( QColor( 0, 255, 0 ) );
		cursorTextPen.setColor( QColor( 255, 0, 0 ) );
	}

	int i, j, c = 1;

#ifndef _DEMO_VERSION_
	QPainter p;
	QFontMetrics fm( p.font() );
	int height = fm.height();
#endif // _DEMO_VERSION_
	p.begin( &m_Pixmap );
		// horizontal line
		p.setPen( linePen );
		for ( i=1 ; i<nNumY ; i++ )
			p.drawLine( 0, nCapY*i, m_Pixmap.width(), nCapY*i );
		// vertical line
		for ( i=1 ; i<nNumX ; i++ )
			p.drawLine( nCapX*i, 0, nCapX*i, m_Pixmap.height() );

		// draw number
		p.setPen( textPen );
		for ( i=0 ; i<nNumY ; i++ )
		{
			for ( j=0 ; j<nNumX ; j++ )
			{
				str.sprintf( "%d", c++ );
				p.drawText( nCapX*j+3, nCapY*i+height+3, str );
			}
		}

		// draw cursor & text
		QRect rtCursor;
		rtCursor.setRect( (m_nCol-1)*UNIT_MONTAGE_SIZE, (m_nRow-1)*m_nUnitHeight,
			UNIT_MONTAGE_SIZE, m_nUnitHeight );
		p.setPen( cursorPen );
		p.drawRect( rtCursor );

		p.setPen( cursorTextPen );
		str.sprintf( "%d", m_nSlice );
		p.drawText( nCapX*(m_nCol-1)+3, nCapY*(m_nRow-1)+height+3, str );
	p.end();

	bitBlt( this, 0, 0, &m_Pixmap );
} /* paintEvent */

// protected
void CMontageViewArea::resizeEvent( QResizeEvent * )
{
} /* resizeEvent */

// protected
void CMontageViewArea::mousePressEvent( QMouseEvent * )
{
} /* mousePressEvent */

// protected
void CMontageViewArea::mouseReleaseEvent( QMouseEvent *e )
{
	calculateRowCol( e->x(), e->y() );
	repaintMontageView();
} /* mouseReleaseEvent */

// protected
void CMontageViewArea::mouseMoveEvent( QMouseEvent * /*e*/ )
{
} /* mouseMoveEvent */

// protected
void CMontageViewArea::keyPressEvent( QKeyEvent *e )
{
	int key = e->key();
	int slice = m_nSlice;

	if ( key == Qt::Key_Left )
	{
		if ( slice != 1 )
			slice--;
		else
			slice = m_nMaxSlice;
	}
	else if ( key == Qt::Key_Right )
	{
		if ( slice != m_nMaxSlice )
			slice++;
		else
			slice = 1;
	}
	else if ( key == Qt::Key_Up )
	{
		if ( (slice-NUM_UNIT_MONTAGE) >= 0 )
			slice -= NUM_UNIT_MONTAGE;
		else
			slice = m_nMaxSlice - (NUM_UNIT_MONTAGE-slice);
	}
	else if ( key == Qt::Key_Down )
	{
		if ( (slice+NUM_UNIT_MONTAGE) <= m_nMaxSlice )
			slice += NUM_UNIT_MONTAGE;
		else
			slice = NUM_UNIT_MONTAGE - (m_nMaxSlice - slice);
	}

	calculateRowCol( slice );
	repaintMontageView();
} /* keyPressEvent */

// private
void CMontageViewArea::calculateRowCol( int slice )
{
	div_t div_result;

	div_result = div( slice, NUM_UNIT_MONTAGE );

	m_nCol = div_result.rem;
	m_nRow = div_result.quot;
	if ( div_result.rem != 0 )
		m_nRow++;
	else
		m_nCol = NUM_UNIT_MONTAGE;

	m_nSlice = m_nCol + (m_nRow-1)*NUM_UNIT_MONTAGE;
} /* calculateRowCol */

void CMontageViewArea::calculateRowCol( int x, int y )
{
	div_t div_result;

	// for x-direction
	div_result = div( x, UNIT_MONTAGE_SIZE );
	m_nCol = div_result.quot;
	if ( div_result.rem != 0 )
		m_nCol++;

	// for y-direction
	div_result = div( y, m_nUnitHeight );
	m_nRow = div_result.quot;
	if ( div_result.rem != 0 )
		m_nRow++;

	m_nSlice = m_nCol + (m_nRow-1)*NUM_UNIT_MONTAGE;
} /* calculateRowCol */

// private
void CMontageViewArea::repaintMontageView()
{
	m_Pixmap = m_OriPixmap;
	repaint( false );

	m_pMontageView->montageMDIWindow()->mdiWindow()->planView()->slotSetSlice( m_nSlice );
} /* repaintMontageView */

// slots
void CMontageViewArea::slotSetSlice( int slice )
{
	calculateRowCol( slice );
	m_Pixmap = m_OriPixmap;
	repaint( false );
} /* slotSetSlice */
